@@ -1,6 +1,7 @@
 # pylint: disable=wrong-import-position
+# pylint: disable=too-many-lines
 #
-# Copyright (C) 2022 Mathieu Comandon <strider@strycore.com>
+# Copyright (C) 2009 Mathieu Comandon <mathieucomandon@gmail.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -39,19 +40,20 @@ from lutris.command import exec_command
 from lutris.database import games as games_db
 from lutris.game import Game, export_game, import_game
 from lutris.installer import get_installers
-from lutris.gui.dialogs.download import simple_downloader
 from lutris.gui.dialogs import ErrorDialog, InstallOrPlayDialog, NoticeDialog, LutrisInitDialog
 from lutris.gui.dialogs.issue import IssueReportWindow
+from lutris.gui.dialogs.delegates import LaunchUIDelegate, InstallUIDelegate, CommandLineUIDelegate
 from lutris.gui.installerwindow import InstallerWindow, InstallationKind
 from lutris.gui.widgets.status_icon import LutrisStatusIcon
 from lutris.migrations import migrate
-from lutris.startup import init_lutris, run_all_checks, StartupRuntimeUpdater
+from lutris.startup import init_lutris, run_all_checks
+from lutris.runtime import RuntimeUpdater
 from lutris.style_manager import StyleManager
 from lutris.util import datapath, log, system
 from lutris.util.http import HTTPError, Request
 from lutris.util.log import logger
 from lutris.util.steam.appmanifest import AppManifest, get_appmanifests
-from lutris.util.steam.config import get_steamapps_paths
+from lutris.util.steam.config import get_steamapps_dirs
 from lutris.services import get_enabled_services
 from lutris.database.services import ServiceGameCollection
 
@@ -75,8 +77,11 @@ class Application(Gtk.Application):
         GObject.add_emission_hook(Game, "game-install-dlc", self.on_game_install_dlc)
 
         GLib.set_application_name(_("Lutris"))
+        self.force_updates = False
         self.css_provider = Gtk.CssProvider.new()
         self.window = None
+        self.launch_ui_delegate = LaunchUIDelegate()
+        self.install_ui_delegate = InstallUIDelegate()
 
         self.running_games = Gio.ListStore.new(Game)
         self.app_windows = {}
@@ -86,7 +91,8 @@ class Application(Gtk.Application):
         self.style_manager = None
 
         if os.geteuid() == 0:
-            NoticeDialog(_("Running Lutris as root is not recommended and may cause unexpected issues"))
+            NoticeDialog(_("Do not run Lutris as root."))
+            sys.exit(2)
 
         try:
             self.css_provider.load_from_path(os.path.join(datapath.get(), "ui", "lutris.css"))
@@ -133,6 +139,14 @@ class Application(Gtk.Application):
             _("Install a game from a yml file"),
             None,
         )
+        self.add_main_option(
+            "force",
+            ord("f"),
+            GLib.OptionFlags.NONE,
+            GLib.OptionArg.NONE,
+            _("Force updates"),
+            None,
+        )
         self.add_main_option(
             "output-script",
             ord("b"),
@@ -197,6 +211,22 @@ class Application(Gtk.Application):
             _("List all known Wine versions"),
             None,
         )
+        self.add_main_option(
+            "list-all-service-games",
+            ord('a'),
+            GLib.OptionFlags.NONE,
+            GLib.OptionArg.NONE,
+            _("List all games for all services in database"),
+            None,
+        )
+        self.add_main_option(
+            "list-service-games",
+            0,
+            GLib.OptionFlags.NONE,
+            GLib.OptionArg.STRING,
+            _("List all games for provided service in database"),
+            None,
+        )
         self.add_main_option(
             "install-runner",
             ord("r"),
@@ -269,7 +299,7 @@ class Application(Gtk.Application):
         signal.signal(signal.SIGINT, signal.SIG_DFL)
 
         action = Gio.SimpleAction.new("quit")
-        action.connect("activate", lambda *x: self.do_shutdown())
+        action.connect("activate", lambda *x: self.quit())
         self.add_action(action)
         self.add_accelerator("<Primary>q", "app.quit")
 
@@ -281,14 +311,15 @@ class Application(Gtk.Application):
             screen = self.window.props.screen  # pylint: disable=no-member
             Gtk.StyleContext.add_provider_for_screen(screen, self.css_provider, Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION)
 
-    @staticmethod
-    def show_update_runtime_dialog():
+    def show_update_runtime_dialog(self, gpu_info):
         if os.environ.get("LUTRIS_SKIP_INIT"):
             logger.debug("Skipping initialization")
         else:
-            runtime_updater = StartupRuntimeUpdater(force=False)
-            init_dialog = LutrisInitDialog(runtime_updater)
-            init_dialog.run()
+            pci_ids = [" ".join([gpu["PCI_ID"], gpu["PCI_SUBSYS_ID"]]) for gpu in gpu_info["gpus"].values()]
+            runtime_updater = RuntimeUpdater(pci_ids=pci_ids, force=self.force_updates)
+            if runtime_updater.has_updates:
+                init_dialog = LutrisInitDialog(runtime_updater)
+                init_dialog.run()
 
     def get_window_key(self, **kwargs):
         if kwargs.get("appid"):
@@ -298,7 +329,7 @@ class Application(Gtk.Application):
         if kwargs.get("installers"):
             return kwargs["installers"][0]["game_slug"]
         if kwargs.get("game"):
-            return str(kwargs["game"].id)
+            return str(kwargs["game"].get_safe_id())
         return str(kwargs)
 
     def show_window(self, window_class, **kwargs):
@@ -358,39 +389,16 @@ class Application(Gtk.Application):
         """Output a script to a file.
         The script is capable of launching a game without the client
         """
+        def on_error(game, error):
+            logger.exception("Unable to generate script: %s", error)
+            return True
+
         game = Game(db_game["id"])
-        game.load_config()
-        game.write_script(script_path)
-
-    def do_command_line(self, command_line):  # noqa: C901  # pylint: disable=arguments-differ
-        # pylint: disable=too-many-locals,too-many-return-statements,too-many-branches
-        # pylint: disable=too-many-statements
-        # TODO: split into multiple methods to reduce complexity (35)
-        options = command_line.get_options_dict()
-
-        # Use stdout to output logs, only if no command line argument is
-        # provided
-        argc = len(sys.argv) - 1
-        if "-d" in sys.argv or "--debug" in sys.argv:
-            argc -= 1
-        if not argc:
-            # Switch back the log output to stderr (the default in Python)
-            # to avoid messing with any output from command line options.
-
-            # Use when targeting Python 3.7 minimum
-            # console_handler.setStream(sys.stderr)
-
-            # Until then...
-            logger.removeHandler(log.console_handler)
-            log.console_handler = logging.StreamHandler(stream=sys.stdout)
-            log.console_handler.setFormatter(log.SIMPLE_FORMATTER)
-            logger.addHandler(log.console_handler)
-
-        # Set up logger
-        if options.contains("debug"):
-            log.console_handler.setFormatter(log.DEBUG_FORMATTER)
-            logger.setLevel(logging.DEBUG)
+        game.connect("game-error", on_error)
+        game.reload_config()
+        game.write_script(script_path, self.launch_ui_delegate)
 
+    def do_handle_local_options(self, options):
         # Text only commands
 
         # Print Lutris version and exit
@@ -399,11 +407,46 @@ class Application(Gtk.Application):
             print(executable_name + "-" + settings.VERSION)
             logger.setLevel(logging.NOTSET)
             return 0
+        return -1  # continue command line processes
 
+    def do_command_line(self, command_line):  # noqa: C901  # pylint: disable=arguments-differ
+        # pylint: disable=too-many-locals,too-many-return-statements,too-many-branches
+        # pylint: disable=too-many-statements
+        # TODO: split into multiple methods to reduce complexity (35)
+        options = command_line.get_options_dict()
+
+        # Use stdout to output logs, only if no command line argument is
+        # provided.
+        argc = len(sys.argv) - 1
+        if "-d" in sys.argv or "--debug" in sys.argv:
+            argc -= 1
+        if not argc:
+            # Switch back the log output to stderr (the default in Python)
+            # to avoid messing with any output from command line options.
+            log.console_handler.setStream(sys.stderr)
+
+        # Set up logger
+        if options.contains("debug"):
+            log.console_handler.setFormatter(log.DEBUG_FORMATTER)
+            logger.setLevel(logging.DEBUG)
+
+        if options.contains("force"):
+            self.force_updates = True
+
+        logger.info("Starting Lutris %s", settings.VERSION)
         init_lutris()
-        migrate()
-        run_all_checks()
 
+        # Perform migrations early if any command line options
+        # might require it to be done, just in case. We migrate
+        # also during the init dialog, but it should be harmless
+        # to do it twice.
+        #
+        # This way, in typical lutris usage, you get to see the
+        # init dialog when migration is happening.
+        if argc:
+            migrate()
+
+        gpu_info = run_all_checks()
         if options.contains("dest"):
             dest_dir = options.lookup_value("dest").get_string()
         else:
@@ -420,6 +463,36 @@ class Application(Gtk.Application):
                 self.print_game_list(command_line, game_list)
             return 0
 
+        # List specified service games
+        if options.contains("list-service-games"):
+            service = options.lookup_value("list-service-games").get_string()
+            game_list = games_db.get_games(filters={"installed": 1, "service": service})
+            service_game_list = ServiceGameCollection.get_for_service(service)
+            for game in service_game_list:
+                game['installed'] = any(('service_id', game['appid']) in item.items() for item in game_list)
+            if options.contains("installed"):
+                service_game_list = [d for d in service_game_list if d['installed']]
+            if options.contains("json"):
+                self.print_service_game_json(command_line, service_game_list)
+            else:
+                self.print_service_game_list(command_line, service_game_list)
+            return 0
+
+        # List all service games
+        if options.contains("list-all-service-games"):
+            game_list = games_db.get_games(filters={"installed": 1})
+            service_game_list = ServiceGameCollection.get_service_games()
+            for game in service_game_list:
+                game['installed'] = any(('service_id', game['appid']) in item.items() for item in game_list if
+                                        item['service'] == game['service'])
+            if options.contains("installed"):
+                service_game_list = [d for d in service_game_list if d['installed']]
+            if options.contains("json"):
+                self.print_service_game_json(command_line, service_game_list)
+            else:
+                self.print_service_game_list(command_line, service_game_list)
+            return 0
+
         # List Steam games
         if options.contains("list-steam-games"):
             self.print_steam_list(command_line)
@@ -436,7 +509,7 @@ class Application(Gtk.Application):
             return 0
 
         # List Wine Runners
-        if options.contains("list-wine-runners"):
+        if options.contains("list-wine-versions"):
             self.print_wine_runners()
             return 0
 
@@ -478,8 +551,8 @@ class Application(Gtk.Application):
             IssueReportWindow(application=self)
             return 0
 
+        url = options.lookup_value(GLib.OPTION_REMAINING)
         try:
-            url = options.lookup_value(GLib.OPTION_REMAINING)
             installer_info = self.get_lutris_action(url)
         except ValueError:
             self._print(command_line, _("%s is not a valid URI") % url.get_strv())
@@ -489,6 +562,9 @@ class Application(Gtk.Application):
         action = installer_info["action"]
         service = installer_info["service"]
         appid = installer_info["appid"]
+        launch_config_name = installer_info["launch_config_name"]
+
+        self.launch_ui_delegate = CommandLineUIDelegate(launch_config_name)
 
         if options.contains("output-script"):
             action = "write-script"
@@ -586,7 +662,7 @@ class Application(Gtk.Application):
 
         if action == "cancel":
             if not self.window.is_visible():
-                self.do_shutdown()
+                self.quit()
             return 0
 
         if action == "install":
@@ -601,12 +677,25 @@ class Application(Gtk.Application):
             if not db_game or not db_game["id"]:
                 logger.warning("No game found in library")
                 if not self.window.is_visible():
-                    self.do_shutdown()
+                    self.quit()
                 return 0
+
+            def on_error(game, error):
+                logger.exception("Unable to launch game: %s", error)
+                return True
+
             game = Game(db_game["id"])
-            game.launch()
+            game.connect("game-error", on_error)
+            game.launch(self.launch_ui_delegate)
+
+            if game.state == game.STATE_STOPPED and not self.window.is_visible():
+                self.quit()
         else:
-            Application.show_update_runtime_dialog()
+            self.show_update_runtime_dialog(gpu_info)
+            # If we're showing the window, it will handle the delegated UI
+            # from here on out, no matter what command line we got.
+            self.launch_ui_delegate = self.window
+            self.install_ui_delegate = self.window
             self.window.present()
             # If the Lutris GUI is started by itself, don't quit it when a game stops
             self.quit_on_game_exit = False
@@ -614,7 +703,7 @@ class Application(Gtk.Application):
 
     @watch_errors(error_result=True)
     def on_game_launch(self, game):
-        game.launch()
+        game.launch(self.launch_ui_delegate)
         return True  # Return True to continue handling the emission hook
 
     @watch_errors(error_result=True)
@@ -629,12 +718,15 @@ class Application(Gtk.Application):
         """Callback to remove the game from the running games"""
         ids = self.get_running_game_ids()
         if str(game.id) in ids:
+            logger.debug("Removing %s from running IDs", game.id)
             try:
                 self.running_games.remove(ids.index(str(game.id)))
             except ValueError:
                 pass
-        else:
+        elif ids:
             logger.warning("%s not in %s", game.id, ids)
+        else:
+            logger.debug("Game has already been removed from running IDs?")
 
         game.emit("game-stopped")
         if settings.read_setting("hide_client_on_game_start") == "True" and not self.quit_on_game_exit:
@@ -642,7 +734,7 @@ class Application(Gtk.Application):
         elif not self.window.is_visible():
             if self.running_games.get_n_items() == 0:
                 if self.quit_on_game_exit or not self.has_tray_icon():
-                    self.do_shutdown()
+                    self.quit()
         return True
 
     @watch_errors(error_result=True)
@@ -661,8 +753,13 @@ class Application(Gtk.Application):
                 game_id = None
 
             if game_id:
+                def on_error(game, error):
+                    logger.exception("Unable to install game: %s", error)
+                    return True
+
                 game = Game(game_id)
-                game.launch()
+                game.connect("game-error", on_error)
+                game.launch(self.launch_ui_delegate)
             return True
         if not game.slug:
             raise ValueError("Invalid game passed: %s" % game)
@@ -703,7 +800,7 @@ class Application(Gtk.Application):
             ids.append(str(game.id))
         return ids
 
-    def get_game_by_id(self, game_id):
+    def get_running_game_by_id(self, game_id):
         for i in range(self.running_games.get_n_items()):
             game = self.running_games.get_item(i)
             if str(game.id) == str(game_id):
@@ -712,11 +809,18 @@ class Application(Gtk.Application):
 
     def on_watched_error(self, error):
         if self.window:
-            ErrorDialog(str(error), parent=self.window)
+            ErrorDialog(error, parent=self.window)
 
     @staticmethod
     def get_lutris_action(url):
-        installer_info = {"game_slug": None, "revision": None, "action": None, "service": None, "appid": None}
+        installer_info = {
+            "game_slug": None,
+            "revision": None,
+            "action": None,
+            "service": None,
+            "appid": None,
+            "launch_config_name": None
+        }
 
         if url:
             url = url.get_strv()
@@ -764,8 +868,34 @@ class Application(Gtk.Application):
         ]
         self._print(command_line, json.dumps(games, indent=2))
 
+    def print_service_game_list(self, command_line, game_list):
+        for game in game_list:
+            self._print(
+                command_line,
+                "{:4} | {:<40} | {:<40} | {:<15} | {:<15}".format(
+                    game["id"],
+                    game["name"][:40],
+                    game["slug"][:40],
+                    game["service"][:15],
+                    "true" if game["installed"] else "false"[:15],
+                ),
+            )
+
+    def print_service_game_json(self, command_line, game_list):
+        games = [
+            {
+                "id": game["id"],
+                "slug": game["slug"],
+                "name": game["name"],
+                "service": game["service"],
+                "installed": game["installed"],
+                "details": game["details"]
+            } for game in game_list
+        ]
+        self._print(command_line, json.dumps(games, indent=2))
+
     def print_steam_list(self, command_line):
-        steamapps_paths = get_steamapps_paths()
+        steamapps_paths = get_steamapps_dirs()
         for path in steamapps_paths if steamapps_paths else []:
             appmanifest_files = get_appmanifests(path)
             for appmanifest_file in appmanifest_files:
@@ -792,15 +922,15 @@ class Application(Gtk.Application):
             monitored_command.stop()
 
     def print_steam_folders(self, command_line):
-        steamapps_paths = get_steamapps_paths()
+        steamapps_paths = get_steamapps_dirs()
         if steamapps_paths:
             for path in steamapps_paths:
                 self._print(command_line, path)
 
     def print_runners(self):
-        runnersName = get_runner_names()
-        sortednames = sorted(runnersName.keys(), key=lambda x: x.lower())
-        for name in sortednames:
+        runner_names = get_runner_names()
+        sorted_names = sorted(runner_names, key=lambda x: x.lower())
+        for name in sorted_names:
             print(name)
 
     def print_wine_runners(self):
@@ -840,7 +970,7 @@ class Application(Gtk.Application):
         else:
             try:
                 runner = import_runner("wine")
-                runner().install(downloader=simple_downloader, version=version)
+                runner().install(self.install_ui_delegate, version=version)
                 print(f"Wine version '{version}' has been installed.")
             except (InvalidRunner, RunnerInstallationError) as ex:
                 print(ex.message)
@@ -855,7 +985,7 @@ class Application(Gtk.Application):
         else:
             print(f"""
 Specified version of Wine is not installed: {version}.
-Please check if the Wine Runner and specified version are installed (for that use --list-wine-runners).
+Please check if the Wine Runner and specified version are installed (for that use --list-wine-versions).
 Also, check that the version specified is in the correct format.
                 """)
 
@@ -869,7 +999,7 @@ Also, check that the version specified is in the correct format.
             if runner.is_installed():
                 print(f"'{runner_name}' is already installed.")
             else:
-                runner.install(version=None, downloader=simple_downloader, callback=None)
+                runner.install(self.install_ui_delegate, version=None, callback=None)
                 print(f"'{runner_name}' has been installed")
         except (InvalidRunner, RunnerInstallationError) as ex:
             print(ex.message)
@@ -897,7 +1027,8 @@ Also, check that the version specified is in the correct format.
     def do_shutdown(self):  # pylint: disable=arguments-differ
         logger.info("Shutting down Lutris")
         if self.window:
-            settings.write_setting("selected_category", self.window.selected_category)
+            selected_category = "%s:%s" % self.window.selected_category
+            settings.write_setting("selected_category", selected_category)
             self.window.destroy()
         Gtk.Application.do_shutdown(self)
 
