@@ -2,6 +2,7 @@ from gettext import gettext as _
 
 from lutris.config import LutrisConfig
 from lutris.gui.config.common import GameDialogCommon
+from lutris.runners import get_runner_human_name
 
 
 class RunnerConfigDialog(GameDialogCommon):
@@ -14,9 +15,11 @@ class RunnerConfigDialog(GameDialogCommon):
         self.lutris_config = LutrisConfig(runner_slug=self.runner_name)
         self.build_notebook()
         self.build_tabs("runner")
-        self.build_action_area(self.on_save)
         self.show_all()
 
+    def get_search_entry_placeholder(self):
+        return _("Search %s options") % get_runner_human_name(self.runner_name)
+
     def on_save(self, wigdet, data=None):
         self.lutris_config.save()
         self.destroy()
