@@ -33,14 +33,14 @@ class InstallerScriptBox(Gtk.VBox):
         runner_label = InstallerLabel("%s" % self.script["runner"])
         runner_label.get_style_context().add_class("info-pill")
         title_box.pack_start(runner_label, False, False, 0)
-        title_box.add(InstallerLabel("<b>%s</b>" % gtk_safe(self.script["version"])))
+        title_box.add(InstallerLabel("<b>%s</b>" % gtk_safe(self.script["version"]), selectable=True))
         title_box.pack_start(InstallerLabel(""), True, True, 0)
-        rating_label = InstallerLabel(self.get_rating())
+        rating_label = InstallerLabel(self.get_rating(), selectable=True)
         rating_label.set_alignment(1, 0.5)
         title_box.pack_end(rating_label, False, False, 0)
         info_box.add(title_box)
         info_box.add(self.get_credits())
-        info_box.add(InstallerLabel(add_url_tags(self.script["description"])))
+        info_box.add(InstallerLabel(add_url_tags(self.script["description"]), selectable=True))
 
         return info_box
 
@@ -61,6 +61,8 @@ class InstallerScriptBox(Gtk.VBox):
 
         install_button = Gtk.Button(_("Install"))
         install_button.connect("clicked", self.on_install_clicked)
+        style_context = install_button.get_style_context()
+        style_context.add_class("suggested-action")
         align.add(install_button)
         return align
 
@@ -78,7 +80,7 @@ class InstallerScriptBox(Gtk.VBox):
         return self._get_installer_label(add_url_tags(credits_text))
 
     def _get_installer_label(self, text):
-        _label = InstallerLabel(text)
+        _label = InstallerLabel(text, selectable=True)
         _label.set_margin_top(12)
         _label.set_margin_bottom(12)
         _label.set_margin_right(12)
