@@ -3,7 +3,8 @@
     COL_ID,
     COL_SLUG,
     COL_NAME,
-    COL_ICON,
+    COL_SORTNAME,
+    COL_MEDIA_PATH,
     COL_YEAR,
     COL_RUNNER,
     COL_RUNNER_HUMAN_NAME,
@@ -15,7 +16,7 @@
     COL_INSTALLED_AT_TEXT,
     COL_PLAYTIME,
     COL_PLAYTIME_TEXT,
-) = list(range(15))
+) = list(range(16))
 
 COLUMN_NAMES = {
     COL_NAME: "name",
