@@ -3,7 +3,6 @@
 import time
 
 from gi.repository import GLib, GObject, Gtk
-from gi.repository.GdkPixbuf import Pixbuf
 
 from lutris import settings
 from lutris.database import sql
@@ -12,8 +11,9 @@ from lutris.gui.views.store_item import StoreItem
 from lutris.util.strings import gtk_safe
 
 from . import (
-    COL_ICON, COL_ID, COL_INSTALLED, COL_INSTALLED_AT, COL_INSTALLED_AT_TEXT, COL_LASTPLAYED, COL_LASTPLAYED_TEXT,
-    COL_NAME, COL_PLATFORM, COL_PLAYTIME, COL_PLAYTIME_TEXT, COL_RUNNER, COL_RUNNER_HUMAN_NAME, COL_SLUG, COL_YEAR
+    COL_ID, COL_INSTALLED, COL_INSTALLED_AT, COL_INSTALLED_AT_TEXT, COL_LASTPLAYED, COL_LASTPLAYED_TEXT, COL_MEDIA_PATH,
+    COL_NAME, COL_PLATFORM, COL_PLAYTIME, COL_PLAYTIME_TEXT, COL_RUNNER, COL_RUNNER_HUMAN_NAME, COL_SLUG, COL_SORTNAME,
+    COL_YEAR
 )
 
 
@@ -39,8 +39,8 @@ def sort_func(model, row1, row2, sort_col):
     value2 = try_lower(value2)
     diff = -1 if value1 < value2 else 0 if value1 == value2 else 1
     if diff == 0:
-        value1 = try_lower(model.get_value(row1, COL_NAME))
-        value2 = try_lower(model.get_value(row2, COL_NAME))
+        value1 = try_lower(model.get_value(row1, COL_SORTNAME))
+        value2 = try_lower(model.get_value(row2, COL_SORTNAME))
         try:
             diff = -1 if value1 < value2 else 0 if value1 == value2 else 1
         except TypeError:
@@ -71,7 +71,8 @@ class GameStore(GObject.Object):
             str,
             str,
             str,
-            Pixbuf,
+            str,
+            str,
             str,
             str,
             str,
@@ -133,10 +134,8 @@ class GameStore(GObject.Object):
         row[COL_ID] = str(store_item.id)
         row[COL_SLUG] = store_item.slug
         row[COL_NAME] = store_item.name
-        if settings.SHOW_MEDIA:
-            row[COL_ICON] = store_item.get_pixbuf()
-        else:
-            row[COL_ICON] = None
+        row[COL_SORTNAME] = store_item.sortname if store_item.sortname else store_item.name
+        row[COL_MEDIA_PATH] = store_item.get_media_path() if settings.SHOW_MEDIA else None
         row[COL_YEAR] = store_item.year
         row[COL_RUNNER] = store_item.runner
         row[COL_RUNNER_HUMAN_NAME] = store_item.runner_text
@@ -158,7 +157,8 @@ class GameStore(GObject.Object):
                 str(game.id),
                 game.slug,
                 game.name,
-                game.get_pixbuf() if settings.SHOW_MEDIA else None,
+                game.sortname if game.sortname else game.name,
+                game.get_media_path() if settings.SHOW_MEDIA else None,
                 game.year,
                 game.runner,
                 game.runner_text,
