@@ -1,12 +1,10 @@
 """Game representation for views"""
 import time
 
+from lutris.database import games
 from lutris.database.games import get_service_games
-from lutris.database.services import ServiceGameCollection
-from lutris.game import Game
-from lutris.gui.widgets.utils import get_pixbuf
-from lutris.runners import RUNNER_NAMES
-from lutris.util import system
+from lutris.runners import get_runner_human_name
+from lutris.services import SERVICES
 from lutris.util.log import logger
 from lutris.util.strings import get_formatted_playtime, gtk_safe
 
@@ -20,6 +18,7 @@ class StoreItem:
         if not game_data:
             raise RuntimeError("No game data provided")
         self._game_data = game_data
+        self._cached_installed_game_data = None
         self.service_media = service_media
 
     def __str__(self):
@@ -28,10 +27,35 @@ class StoreItem:
     def __repr__(self):
         return "<Store id=%s slug=%s>" % (self.id, self.slug)
 
+    @property
+    def _installed_game_data(self):
+        """Provides- and caches- the DB data for the installed game corresponding to this one,
+        if it's a service game. We can get away with caching this because StoreItem instances are
+        very short-lived, so the game won't be changed underneath us."""
+        appid = self._game_data.get("appid")
+        if appid:
+            if self._cached_installed_game_data is None:
+                self._cached_installed_game_data = games.get_game_for_service(self.service,
+                                                                              self._game_data["appid"]) or {}
+            return self._cached_installed_game_data
+
+        return None
+
+    def _get_game_attribute(self, key):
+        value = self._game_data.get(key)
+
+        if not value:
+            game_data = self._installed_game_data
+
+            if game_data:
+                value = game_data.get(key)
+
+        return value
+
     @property
     def id(self):  # pylint: disable=invalid-name
         """Game internal ID"""
-        # Return an unique identifier for the game.
+        # Return a unique identifier for the game.
         # Since service games are not related to lutris, use the appid
         if "service_id" not in self._game_data:
             if "appid" in self._game_data:
@@ -54,29 +78,38 @@ class StoreItem:
         """Name"""
         return gtk_safe(self._game_data["name"])
 
+    @property
+    def sortname(self):
+        """Name used for sorting"""
+        return gtk_safe(self._get_game_attribute("sortname") or "")
+
     @property
     def year(self):
         """Year"""
-        return str(self._game_data.get("year") or "")
+        return str(self._get_game_attribute("year") or "")
 
     @property
     def runner(self):
         """Runner slug"""
-        return gtk_safe(self._game_data.get("runner")) or ""
+        _runner = self._get_game_attribute("runner")
+        return gtk_safe(_runner) or ""
 
     @property
     def runner_text(self):
         """Runner name"""
-        return gtk_safe(RUNNER_NAMES.get(self.runner))
+        return gtk_safe(get_runner_human_name(self.runner))
 
     @property
     def platform(self):
         """Platform"""
-        _platform = self._game_data.get("platform")
-        if not _platform and not self.service and self.installed:
-            game_inst = Game(self._game_data["id"])
-            if game_inst.platform:
-                _platform = game_inst.platform
+        _platform = self._get_game_attribute("platform")
+
+        if not _platform and self.service in SERVICES:
+            service = SERVICES[self.service]()
+            _platforms = service.get_game_platforms(self._game_data)
+            if _platforms:
+                _platform = ", ".join(_platforms)
+
         return gtk_safe(_platform)
 
     @property
@@ -88,32 +121,17 @@ class StoreItem:
             return False
         return self._game_data.get("installed")
 
-    def get_pixbuf(self):
-        """Pixbuf varying on icon type"""
+    def get_media_path(self):
+        """Returns the path to the image file for this item"""
         if self._game_data.get("icon"):
-            image_path = self._game_data["icon"]
-        else:
-            image_path = self.service_media.get_absolute_path(self.slug)
-            if not system.path_exists(image_path):
-                service = self._game_data.get("service")
-                appid = self._game_data.get("service_id")
-                if appid:
-                    service_game = ServiceGameCollection.get_game(service, appid)
-                else:
-                    service_game = None
-                if service_game:
-                    image_path = self.service_media.get_absolute_path(service_game["slug"])
-        if system.path_exists(image_path):
-            return get_pixbuf(image_path, self.service_media.size, is_installed=self.installed)
-        return self.service_media.get_pixbuf_for_game(
-            self._game_data["slug"],
-            is_installed=self.installed
-        )
+            return self._game_data["icon"]
+
+        return self.service_media.get_media_path(self.slug)
 
     @property
     def installed_at(self):
         """Date of install"""
-        return self._game_data.get("installed_at")
+        return self._get_game_attribute("installed_at")
 
     @property
     def installed_at_text(self):
@@ -126,7 +144,7 @@ class StoreItem:
     @property
     def lastplayed(self):
         """Date of last play"""
-        return self._game_data.get("lastplayed")
+        return self._get_game_attribute("lastplayed")
 
     @property
     def lastplayed_text(self):
@@ -142,7 +160,7 @@ class StoreItem:
     def playtime(self):
         """Playtime duration in hours"""
         try:
-            return float(self._game_data.get("playtime", 0))
+            return float(self._get_game_attribute("playtime") or 0)
         except (TypeError, ValueError):
             return 0.0
 
