@@ -1,16 +1,16 @@
 """Misc widgets used in the GUI."""
 # Standard Library
 import os
+import shlex
+import urllib.parse
 from gettext import gettext as _
 
 # Third Party Libraries
-from gi.repository import GObject, Gtk, Pango
+from gi.repository import GLib, GObject, Gtk, Pango
 
 # Lutris Modules
-from lutris.gui.widgets.utils import get_stock_icon
 from lutris.util import system
 from lutris.util.linux import LINUX_SYSTEM
-from lutris.util.log import logger
 
 
 class SlugEntry(Gtk.Entry, Gtk.Editable):
@@ -35,20 +35,24 @@ class NumberEntry(Gtk.Entry, Gtk.Editable):
 
 
 class FileChooserEntry(Gtk.Box):
-
     """Editable entry with a file picker button"""
 
     max_completion_items = 15  # Maximum number of items to display in the autocompletion dropdown.
 
+    __gsignals__ = {
+        "changed": (GObject.SIGNAL_RUN_FIRST, None, ()),
+    }
+
     def __init__(
         self,
         title=_("Select file"),
         action=Gtk.FileChooserAction.OPEN,
-        path=None,
+        text=None,
         default_path=None,
         warn_if_non_empty=False,
         warn_if_ntfs=False,
         activates_default=False,
+        shell_quoting=False
     ):
         super().__init__(
             orientation=Gtk.Orientation.VERTICAL,
@@ -57,19 +61,23 @@ class FileChooserEntry(Gtk.Box):
         )
         self.title = title
         self.action = action
-        self.path = os.path.expanduser(path) if path else None
-        self.default_path = os.path.expanduser(default_path) if default_path else path
         self.warn_if_non_empty = warn_if_non_empty
         self.warn_if_ntfs = warn_if_ntfs
+        self.shell_quoting = shell_quoting
 
         self.path_completion = Gtk.ListStore(str)
 
         self.entry = Gtk.Entry(visible=True)
+        self.set_text(text)  # do before set up signal handlers
+        self.original_text = self.get_text()
+        self.default_path = os.path.expanduser(default_path) if default_path else self.get_path()
+
         self.entry.set_activates_default(activates_default)
         self.entry.set_completion(self.get_completion())
         self.entry.connect("changed", self.on_entry_changed)
-        if path:
-            self.entry.set_text(path)
+        self.entry.connect("activate", self.on_activate)
+        self.entry.connect("focus-out-event", self.on_focus_out)
+        self.entry.connect("backspace", self.on_backspace)
 
         browse_button = Gtk.Button(_("Browse..."), visible=True)
         browse_button.connect("clicked", self.on_browse_clicked)
@@ -79,14 +87,49 @@ class FileChooserEntry(Gtk.Box):
         box.add(browse_button)
         self.pack_start(box, False, False, 0)
 
+    def set_text(self, text):
+        if self.shell_quoting and text:
+            command_array = shlex.split(text)
+            if command_array:
+                expanded = os.path.expanduser(command_array[0])
+                command_array[0] = expanded
+                rejoined = shlex.join(command_array)
+                self.original_text = rejoined
+                self.entry.set_text(rejoined)
+                return
+
+        expanded = os.path.expanduser(text) if text else ""
+        self.original_text = expanded
+        self.entry.set_text(expanded)
+
+    def set_path(self, path):
+        if self.shell_quoting:
+            command_array = shlex.split(self.get_text())
+            if command_array:
+                command_array[0] = os.path.expanduser(path) if path else ""
+                rejoined = shlex.join(command_array)
+                self.original_text = rejoined
+                self.entry.set_text(rejoined)
+                return
+
+        expanded = os.path.expanduser(path) if path else ""
+        self.original_text = expanded
+        self.entry.set_text(expanded)
+
     def get_text(self):
-        """Return the entry's text"""
+        """Return the entry's text. If shell_quoting is one, this is actually a command
+        line (with argument quoting) and not a simple path."""
         return self.entry.get_text()
 
-    def get_filename(self):
-        """Deprecated"""
-        logger.warning("Just use get_text")
-        return self.get_text()
+    def get_path(self):
+        """Returns the path in the entry; if shell_quoting is set, this extracts
+        the command from the text and returns only that."""
+        text = self.get_text()
+        if self.shell_quoting:
+            command_array = shlex.split(text)
+            return command_array[0] if command_array else ""
+
+        return text
 
     def get_completion(self):
         """Return an EntryCompletion widget"""
@@ -104,7 +147,7 @@ class FileChooserEntry(Gtk.Box):
 
     def get_default_folder(self):
         """Return the default folder for the file picker"""
-        default_path = self.path or self.default_path or ""
+        default_path = self.get_path() or self.default_path or ""
         if not default_path or not system.path_exists(default_path):
             current_entry = self.get_text()
             if system.path_exists(current_entry):
@@ -120,29 +163,43 @@ class FileChooserEntry(Gtk.Box):
 
         if response == Gtk.ResponseType.ACCEPT:
             target_path = file_chooser_dialog.get_filename()
-            if target_path:
-                self.entry.set_text(system.reverse_expanduser(target_path))
+
+            if target_path and self.shell_quoting:
+                command_array = shlex.split(self.entry.get_text())
+                text = shlex.join([target_path] + command_array[1:])
+            else:
+                text = target_path
+
+            self.original_text = text
+            self.entry.set_text(text)
 
         file_chooser_dialog.destroy()
 
     def on_entry_changed(self, widget):
         """Entry changed callback"""
         self.clear_warnings()
-        path = widget.get_text()
-        if not path:
-            return
-        path = os.path.expanduser(path)
-        self.update_completion(path)
-        self.path = path
+
+        # If the user isn't editing this entry, we'll apply updates
+        # immediately upon any change
+
+        if not self.entry.has_focus():
+            if self.normalize_path():
+                # We changed the text on commit, so we return here to avoid a double changed signal
+                return
+
+        text = self.get_text()
+        path = self.get_path()
+        self.original_text = text
+
         if self.warn_if_ntfs and LINUX_SYSTEM.get_fs_type_for_path(path) == "ntfs":
             ntfs_box = Gtk.Box(spacing=6, visible=True)
             warning_image = Gtk.Image(visible=True)
-            warning_image.set_from_pixbuf(get_stock_icon("dialog-warning", 32))
+            warning_image.set_from_icon_name("dialog-warning", Gtk.IconSize.DND)
             ntfs_box.add(warning_image)
             ntfs_label = Gtk.Label(visible=True)
             ntfs_label.set_markup(_(
                 "<b>Warning!</b> The selected path is located on a drive formatted by Windows.\n"
-                "Games and programs installed on Windows drives usually <b>don't work</b>."
+                "Games and programs installed on Windows drives <b>don't work</b>."
             ))
             ntfs_box.add(ntfs_label)
             self.pack_end(ntfs_box, False, False, 10)
@@ -150,7 +207,7 @@ class FileChooserEntry(Gtk.Box):
             non_empty_label = Gtk.Label(visible=True)
             non_empty_label.set_markup(_(
                 "<b>Warning!</b> The selected path "
-                "contains files. Installation might not work properly."
+                "contains files. Installation will not work properly."
             ))
             self.pack_end(non_empty_label, False, False, 10)
         parent = system.get_existing_parent(path)
@@ -162,6 +219,46 @@ class FileChooserEntry(Gtk.Box):
             ))
             self.pack_end(non_writable_destination_label, False, False, 10)
 
+        self.emit("changed")
+
+    def on_activate(self, _widget):
+        self.normalize_path()
+        self.detect_changes()
+
+    def on_focus_out(self, _widget, _event):
+        self.normalize_path()
+        self.detect_changes()
+
+    def on_backspace(self, _widget):
+        GLib.idle_add(self.detect_changes)
+
+    def detect_changes(self):
+        """Detects if the text has changed and updates self.original_text and fires
+        the changed signal. Lame, but Gtk.Entry does not always fire its
+        changed event when edited!"""
+        new_text = self.get_text()
+        if self.original_text != new_text:
+            self.original_text = new_text
+            self.emit("changed")
+        return False  # used as idle function
+
+    def normalize_path(self):
+        original_path = self.get_path()
+        path = original_path.strip("\r\n")
+
+        if path.startswith('file:///'):
+            path = urllib.parse.unquote(path[len('file://'):])
+
+        path = os.path.expanduser(path)
+
+        self.update_completion(path)
+
+        if path != original_path:
+            self.set_path(path)
+            return True
+
+        return False
+
     def update_completion(self, current_path):
         """Update the auto-completion widget with the current path"""
         self.path_completion.clear()
@@ -193,29 +290,17 @@ class FileChooserEntry(Gtk.Box):
 class Label(Gtk.Label):
     """Standardised label for config vboxes."""
 
-    def __init__(self, message=None):
+    def __init__(self, message=None, width_request=230):
         """Custom init of label."""
         super().__init__(label=message)
         self.set_line_wrap(True)
         self.set_max_width_chars(22)
         self.set_line_wrap_mode(Pango.WrapMode.WORD_CHAR)
-        self.set_size_request(230, -1)
+        self.set_size_request(width_request, -1)
         self.set_alignment(0, 0.5)
         self.set_justify(Gtk.Justification.LEFT)
 
 
-class InstallerLabel(Gtk.Label):
-    """Label for installer window"""
-
-    def __init__(self, message=None):
-        super().__init__(label=message)
-        self.set_max_width_chars(80)
-        self.set_property("wrap", True)
-        self.set_use_markup(True)
-        self.set_selectable(True)
-        self.set_alignment(0.5, 0)
-
-
 class VBox(Gtk.Box):
     def __init__(self, **kwargs):
         super().__init__(orientation=Gtk.Orientation.VERTICAL, margin_top=18, **kwargs)
