@@ -17,7 +17,7 @@ from lutris.util.wine.cabinstall import CabInstaller
 from lutris.util.wine.prefix import WinePrefixManager
 from lutris.util.wine.wine import (
     WINE_DEFAULT_ARCH, WINE_DIR, detect_arch, detect_prefix_arch, get_overrides_env, get_real_executable,
-    use_lutris_runtime
+    is_installed_systemwide
 )
 
 
@@ -95,6 +95,7 @@ def create_prefix(  # noqa: C901
         overrides = {}
     if not prefix:
         raise ValueError("No Wine prefix path given")
+    prefix = os.path.expanduser(prefix)
     logger.info("Creating a %s prefix in %s", arch, prefix)
 
     # Follow symlinks, don't delete existing ones as it would break some setups
@@ -196,6 +197,20 @@ def winekill(prefix, arch=WINE_DEFAULT_ARCH, wine_path=None, env=None, initial_p
     logger.debug("Done waiting.")
 
 
+def use_lutris_runtime(wine_path, force_disable=False):
+    """Returns whether to use the Lutris runtime.
+    The runtime can be forced to be disabled, otherwise
+    it's disabled automatically if Wine is installed system wide.
+    """
+    if force_disable or runtime.RUNTIME_DISABLED:
+        return False
+    if WINE_DIR in wine_path:
+        return True
+    if is_installed_systemwide():
+        return False
+    return True
+
+
 # pragma pylint: disable=too-many-locals
 def wineexec(  # noqa: C901
     executable,
@@ -247,7 +262,7 @@ def wineexec(  # noqa: C901
         exclude_processes = shlex.split(exclude_processes)
 
     if not runner:
-        runner = import_runner("wine")()
+        runner = import_runner("wine")(prefix=prefix, working_dir=working_dir, wine_arch=arch)
 
     if not wine_path:
         wine_path = runner.get_executable()
@@ -326,6 +341,29 @@ def wineexec(  # noqa: C901
 # pragma pylint: enable=too-many-locals
 
 
+def find_winetricks(env=None, system_winetricks=False):
+    """Find winetricks path."""
+    winetricks_path = os.path.join(settings.RUNTIME_DIR, "winetricks/winetricks")
+    if system_winetricks or not system.path_exists(winetricks_path):
+        winetricks_path = system.find_executable("winetricks")
+        working_dir = None
+        if not winetricks_path:
+            raise RuntimeError("No installation of winetricks found")
+    else:
+        # We will use our own zenity if available, which is here, and it
+        # also needs a data file in this directory. We have to set the
+        # working_dir, so it will find the data file.
+        working_dir = os.path.join(settings.RUNTIME_DIR, "winetricks")
+
+        if not env:
+            env = {}
+
+        path = env.get("PATH", os.environ["PATH"])
+        env["PATH"] = "%s:%s" % (working_dir, path)
+
+    return (winetricks_path, working_dir, env)
+
+
 def winetricks(
     app,
     prefix=None,
@@ -339,23 +377,7 @@ def winetricks(
     runner=None
 ):
     """Execute winetricks."""
-    winetricks_path = os.path.join(settings.RUNTIME_DIR, "winetricks/winetricks")
-    if system_winetricks or not system.path_exists(winetricks_path):
-        winetricks_path = system.find_executable("winetricks")
-        working_dir = None
-        if not winetricks_path:
-            raise RuntimeError("No installation of winetricks found")
-    else:
-        # We will use our own zentiy if available, which is here and it
-        # also needs a data file in this directory. We have to set the
-        # working_dir so it will find the data file.
-        working_dir = os.path.join(settings.RUNTIME_DIR, "winetricks")
-
-        if not env:
-            env = {}
-
-        path = env.get("PATH", os.environ["PATH"])
-        env["PATH"] = "%s:%s" % (working_dir, path)
+    winetricks_path, working_dir, env = find_winetricks(env, system_winetricks)
 
     if wine_path:
         winetricks_wine = wine_path
@@ -419,14 +441,22 @@ def install_cab_component(cabfile, component, wine_path=None, prefix=None, arch=
     cab_installer.cleanup()
 
 
-def open_wine_terminal(terminal, wine_path, prefix, env):
+def open_wine_terminal(terminal, wine_path, prefix, env, system_winetricks):
+    winetricks_path, _working_dir, env = find_winetricks(env, system_winetricks)
     aliases = {
         "wine": wine_path,
         "winecfg": wine_path + "cfg",
         "wineserver": wine_path + "server",
         "wineboot": wine_path + "boot",
+        "winetricks": winetricks_path,
     }
     env["WINEPREFIX"] = prefix
+    # Ensure scripts you run see the desired version of WINE too
+    # by putting it on the PATH.
+    wine_directory = os.path.split(wine_path)[0]
+    if wine_directory:
+        path = env.get("PATH", os.environ["PATH"])
+        env["PATH"] = "%s:%s" % (wine_directory, path)
     shell_command = get_shell_command(prefix, env, aliases)
     terminal = terminal or linux.get_default_terminal()
     system.execute([terminal, "-e", shell_command])
