@@ -124,14 +124,15 @@ def scan_dir_for_bios(path):
 class fsuae(Runner):
     human_name = _("FS-UAE")
     description = _("Amiga emulator")
+    flatpak_id = "net.fsuae.FS-UAE"
     platforms = [
         AMIGAS["A500"]["name"],
         AMIGAS["A500+"]["name"],
         AMIGAS["A600"]["name"],
-        AMIGAS["A1000"]["name"],
-        AMIGAS["A1200"]["name"],
         AMIGAS["A1200"]["name"],
+        AMIGAS["A3000"]["name"],
         AMIGAS["A4000"]["name"],
+        AMIGAS["A1000"]["name"],
         AMIGAS["CD32"]["name"],
         AMIGAS["CDTV"]["name"],
     ]
@@ -230,14 +231,17 @@ class fsuae(Runner):
                 "Files ending in .hdf will be mounted as hard drives and "
                 "ISOs can be used for Amiga CD32 and CDTV models."
             ),
-        }, {
+        },
+        {
             "option": "disks",
+            "section": _("Media"),
             "type": "multiple",
             "label": _("Additionnal floppies"),
             "default_path": "game_path",
             "help": _("The additional floppy disk image(s)."),
         }, {
             "option": "cdrom_image",
+            "section": _("Media"),
             "label": _("CD-ROM image"),
             "type": "file",
             "help": _("CD-ROM image to use on non CD32/CDTV models")
@@ -255,6 +259,7 @@ class fsuae(Runner):
         },
         {
             "option": "kickstart_file",
+            "section": _("Kickstart"),
             "label": _("Kickstart ROMs location"),
             "type": "file",
             "help": _(
@@ -267,6 +272,7 @@ class fsuae(Runner):
         },
         {
             "option": "kickstart_ext_file",
+            "section": _("Kickstart"),
             "label": _("Extended Kickstart location"),
             "type": "file",
             "advanced": True,
@@ -274,18 +280,46 @@ class fsuae(Runner):
         },
         {
             "option": "gfx_fullscreen_amiga",
+            "section": _("Graphics"),
             "label": _("Fullscreen (F12 + S to switch)"),
             "type": "bool",
             "default": False,
         },
         {
             "option": "scanlines",
+            "section": _("Graphics"),
             "label": _("Scanlines display style"),
             "type": "bool",
             "default": False,
             "help": _("Activates a display filter adding scanlines to imitate "
                       "the displays of yesteryear."),
         },
+        {
+            "option": "grafixcard",
+            "section": _("Graphics"),
+            "label": _("Graphics Card"),
+            "type": "choice",
+            "choices": gpucard_choices,
+            "default": "None",
+            "advanced": True,
+            "help": _(
+                "Use this option to enable a graphics card. This option is none by default, in "
+                "which case only chipset graphics (OCS/ECS/AGA) support is available."
+            )
+        },
+        {
+            "option": "grafixmemory",
+            "section": _("Graphics"),
+            "label": _("Graphics Card RAM"),
+            "type": "choice",
+            "choices": gpumem_choices,
+            "default": "0",
+            "advanced": True,
+            "help": _(
+                "Override the amount of graphics memory on the graphics card. The 0 MB option is "
+                "not really valid, but exists for user interface reasons."
+            )
+        },
         {
             "option": "cpumodel",
             "label": _("CPU"),
@@ -319,6 +353,7 @@ class fsuae(Runner):
         },
         {
             "option": "fdvolume",
+            "section": _("Media"),
             "label": _("Floppy Drive Volume"),
             "type": "choice",
             "choices": flsound_choices,
@@ -329,6 +364,7 @@ class fsuae(Runner):
         },
         {
             "option": "fdspeed",
+            "section": _("Media"),
             "label": _("Floppy Drive Speed"),
             "type": "choice",
             "choices": flspeed_choices,
@@ -341,30 +377,6 @@ class fsuae(Runner):
                 "all floppy operations complete immediately. The default is 100 for most models."
             )
         },
-        {
-            "option": "grafixcard",
-            "label": _("Graphics Card"),
-            "type": "choice",
-            "choices": gpucard_choices,
-            "default": "None",
-            "advanced": True,
-            "help": _(
-                "Use this option to enable a graphics card. This option is none by default, in "
-                "which case only chipset graphics (OCS/ECS/AGA) support is available."
-            )
-        },
-        {
-            "option": "grafixmemory",
-            "label": _("Graphics Card RAM"),
-            "type": "choice",
-            "choices": gpumem_choices,
-            "default": "0",
-            "advanced": True,
-            "help": _(
-                "Override the amount of graphics memory on the graphics card. The 0 MB option is "
-                "not really valid, but exists for user interface reasons."
-            )
-        },
         {
             "option": "jitcompiler",
             "label": _("JIT Compiler"),
@@ -388,8 +400,8 @@ class fsuae(Runner):
             "default": False,
             "advanced": True,
             "help":
-            _("Warn if running with a CPU governor other than performance. "
-              "Set to true to disable the warning.")
+                _("Warn if running with a CPU governor other than performance. "
+                  "Set to true to disable the warning.")
         },
         {
             "option": "bsdsocket",
@@ -486,4 +498,4 @@ class fsuae(Runner):
         return params
 
     def play(self):
-        return {"command": [self.get_executable()] + self.get_params() + self.insert_floppies()}
+        return {"command": self.get_command() + self.get_params() + self.insert_floppies()}
