@@ -7,6 +7,41 @@ from lutris.runners.runner import Runner
 from lutris.util import system
 from lutris.util.strings import split_arguments
 
+_supported_scale_factors = {
+    "hq": ["2", "3"],
+    "edge": ["2", "3"],
+    "advmame": ["2", "3"],
+    "sai": ["2"],
+    "supersai": ["2"],
+    "supereagle": ["2"],
+    "dotmatrix": ["2"],
+    "tv2x": ["2"],
+}
+
+
+def _get_opengl_warning(config, _option_key):
+    if "scaler" in config and "renderer" in config:
+        renderer = config["renderer"]
+        if renderer and renderer != "software":
+            scaler = config["scaler"]
+            if scaler and scaler != "normal":
+                return _("<b>Warning</b> Scalers may not work with OpenGL rendering.")
+
+    return None
+
+
+def _get_scale_factor_warning(config, _option_key):
+    """Generate a warning message for when the scaler and scale-factor can't be used together."""
+    if "scaler" in config and "scale-factor" in config:
+        scaler = config["scaler"]
+        if scaler in _supported_scale_factors:
+            scale_factor = config["scale-factor"]
+            if scale_factor not in _supported_scale_factors[scaler]:
+                return _("<b>Warning</b> The '%s' scaler does not work with a scale factor of %s.") % (
+                    scaler, scale_factor)
+
+    return None
+
 
 class scummvm(Runner):
     description = _("Engine for point-and-click games.")
@@ -14,6 +49,7 @@ class scummvm(Runner):
     platforms = [_("Linux")]
     runnable_alone = True
     runner_executable = "scummvm/bin/scummvm"
+    flatpak_id = "org.scummvm.ScummVM"
     game_options = [
         {
             "option": "game_id",
@@ -37,9 +73,11 @@ class scummvm(Runner):
         "aspect": "--aspect-ratio",
         "subtitles": "--subtitles",
         "fullscreen": "--fullscreen",
-        "gfx-mode": "--gfx-mode=%s",
+        "scaler": "--scaler=%s",
         "scale-factor": "--scale-factor=%s",
+        "renderer": "--renderer=%s",
         "render-mode": "--render-mode=%s",
+        "stretch-mode": "--stretch-mode=%s",
         "filtering": "--filtering",
         "platform": "--platform=%s",
         "engine-speed": "--engine-speed=%s",
@@ -73,12 +111,14 @@ class scummvm(Runner):
     runner_options = [
         {
             "option": "fullscreen",
+            "section": _("Graphics"),
             "label": _("Fullscreen"),
             "type": "bool",
             "default": True,
         },
         {
             "option": "subtitles",
+            "section": _("Graphics"),
             "label": _("Enable subtitles"),
             "type": "bool",
             "default": False,
@@ -86,6 +126,7 @@ class scummvm(Runner):
         },
         {
             "option": "aspect",
+            "section": _("Graphics"),
             "label": _("Aspect ratio correction"),
             "type": "bool",
             "default": True,
@@ -97,67 +138,107 @@ class scummvm(Runner):
             ),
         },
         {
-            "option": "gfx-mode",
+            "option": "scaler",
+            "section": _("Graphics"),
             "label": _("Graphic scaler"),
             "type": "choice",
-            "default": "3x",
+            "default": "normal",
             "choices": [
-                ("1x", "1x"),
-                ("2x", "2x"),
-                ("3x", "3x"),
-                ("hq2x", "hq2x"),
-                ("hq3x", "hq3x"),
-                ("advmame2x", "advmame2x"),
-                ("advmame3x", "advmame3x"),
-                ("2xsai", "2xsai"),
-                ("super2xsai", "super2xsai"),
+                ("normal", "normal"),
+                ("hq", "hq"),
+                ("edge", "edge"),
+                ("advmame", "advmame"),
+                ("sai", "sai"),
+                ("supersai", "supersai"),
                 ("supereagle", "supereagle"),
-                ("tv2x", "tv2x"),
+                ("pm", "pm"),
                 ("dotmatrix", "dotmatrix"),
+                ("tv2x", "tv2x"),
+            ],
+            "warning": _get_opengl_warning,
+            "help":
+                _("The algorithm used to scale up the game's base "
+                  "resolution, resulting in different visual styles. "),
+        },
+        {
+            "option": "scale-factor",
+            "section": _("Graphics"),
+            "label": _("Scale factor"),
+            "type": "choice",
+            "default": "3",
+            "choices": [
+                ("1", "1"),
+                ("2", "2"),
+                ("3", "3"),
+                ("4", "4"),
+                ("5", "5"),
             ],
             "help":
-            _("The algorithm used to scale up the game's base "
-              "resolution, resulting in different visual styles. "),
+                _("Changes the resolution of the game. "
+                  "For example, a 2x scale will take a 320x200 "
+                  "resolution game and scale it up to 640x400. "),
+            "warning": _get_scale_factor_warning
+        },
+        {
+            "option": "renderer",
+            "section": _("Graphics"),
+            "label": _("Renderer"),
+            "type": "choice",
+            "choices": [
+                (_("Auto"), ""),
+                (_("Software"), "software"),
+                (_("OpenGL"), "opengl"),
+                (_("OpenGL (with shaders)"), "opengl_shaders")
+            ],
+            "default": "",
+            "advanced": True,
+            "help": _("Changes the rendering method used for 3D games."),
         },
-        # {
-        #    "option": "scale-factor",
-        #    "label": _("Scaler factor"),
-        #    "type": "choice",
-        #    "choices": [
-        #        ("1", "1"),
-        #        ("2", "2"),
-        #        ("3", "3"),
-        #        ("4", "4"),
-        #        ("5", "5"),
-        #    ],
-        #    "help":
-        #    _("Changes the resolution of the game. "
-        #      "For example, a 2x scaler will take a 320x200 "
-        #      "resolution game and scale it up to 640x400. "),
-        # },
         {
             "option": "render-mode",
+            "section": _("Graphics"),
             "label": _("Render mode"),
             "type": "choice",
             "choices": [
-                ("hercGreen", "hercGreen"),
-                ("hercAmber", "hercAmber"),
-                ("cga", "cga"),
-                ("ega", "ega"),
-                ("vga", "vga"),
-                ("amiga", "amiga"),
-                ("fmtowns", "fmtowns"),
-                ("pc9821", "pc9821"),
-                ("pc9801", "pc9801"),
-                ("2gs", "2gs"),
-                ("atari", "atari"),
-                ("macintosh", "macintosh"),
+                (_("Auto"), ""),
+                (_("Hercules (Green)"), "hercGreen"),
+                (_("Hercules (Amber)"), "hercAmber"),
+                (_("CGA"), "cga"),
+                (_("EGA"), "ega"),
+                (_("VGA"), "vga"),
+                (_("Amiga"), "amiga"),
+                (_("FM Towns"), "fmtowns"),
+                (_("PC-9821"), "pc9821"),
+                (_("PC-9801"), "pc9801"),
+                (_("Apple IIgs"), "2gs"),
+                (_("Atari ST"), "atari"),
+                (_("Macintosh"), "macintosh"),
             ],
+            "default": "",
             "advanced": True,
-            "help": _("Changes how the game is rendered."),
+            "help": _("Changes the graphics hardware the game will target, if the game supports this."),
+        },
+        {
+            "option": "stretch-mode",
+            "section": _("Graphics"),
+            "label": _("Stretch mode"),
+            "type": "choice",
+            "choices": [
+                (_("Auto"), ""),
+                (_("Center"), "center"),
+                (_("Pixel Perfect"), "pixel-perfect"),
+                (_("Even Pixels"), "even-pixels"),
+                (_("Stretch"), "stretch"),
+                (_("Fit"), "fit"),
+                (_("Fit (force aspect ratio)"), "fit_force_aspect"),
+            ],
+            "default": "",
+            "advanced": True,
+            "help": _("Changes how the game is placed when the window is resized."),
         },
         {
             "option": "filtering",
+            "section": _("Graphics"),
             "label": _("Filtering"),
             "type": "bool",
             "help": _("Uses bilinear interpolation instead of nearest neighbor "
@@ -212,6 +293,7 @@ class scummvm(Runner):
         {
             "option": "music-tempo",
             "type": "string",
+            "section": _("Audio"),
             "label": _("Music tempo"),
             "help": _("Sets music tempo (in percent, 50-200) for SCUMM games (default: 100)"),
             "advanced": True,
@@ -219,12 +301,14 @@ class scummvm(Runner):
         {
             "option": "dimuse-tempo",
             "type": "string",
+            "section": _("Audio"),
             "label": _("Digital iMuse tempo"),
             "help": _("Sets internal Digital iMuse tempo (10 - 100) per second (default: 10)"),
             "advanced": True,
         },
         {
             "option": "music-driver",
+            "section": _("Audio"),
             "label": _("Music driver"),
             "type": "choice",
             "choices": [
@@ -246,6 +330,7 @@ class scummvm(Runner):
         },
         {
             "option": "output-rate",
+            "section": _("Audio"),
             "label": _("Output rate"),
             "type": "choice",
             "choices": [
@@ -258,6 +343,7 @@ class scummvm(Runner):
         },
         {
             "option": "opl-driver",
+            "section": _("Audio"),
             "label": _("OPL driver"),
             "type": "choice",
             "choices": [
@@ -277,6 +363,7 @@ class scummvm(Runner):
         {
             "option": "music-volume",
             "type": "string",
+            "section": _("Audio"),
             "label": _("Music volume"),
             "help": _("Sets the music volume, 0-255 (default: 192)"),
             "advanced": True,
@@ -284,6 +371,7 @@ class scummvm(Runner):
         {
             "option": "sfx-volume",
             "type": "string",
+            "section": _("Audio"),
             "label": _("SFX volume"),
             "help": _("Sets the sfx volume, 0-255 (default: 192)"),
             "advanced": True,
@@ -291,6 +379,7 @@ class scummvm(Runner):
         {
             "option": "speech-volume",
             "type": "string",
+            "section": _("Audio"),
             "label": _("Speech volume"),
             "help": _("Sets the speech volume, 0-255 (default: 192)"),
             "advanced": True,
@@ -298,12 +387,14 @@ class scummvm(Runner):
         {
             "option": "midi-gain",
             "type": "string",
+            "section": _("Audio"),
             "label": _("MIDI gain"),
             "help": _("Sets the gain for MIDI playback. 0-1000 (default: 100)"),
             "advanced": True,
         },
         {
             "option": "soundfont",
+            "section": _("Audio"),
             "type": "string",
             "label": _("Soundfont"),
             "help": _("Specifies the path to a soundfont file."),
@@ -311,6 +402,7 @@ class scummvm(Runner):
         },
         {
             "option": "multi-midi",
+            "section": _("Audio"),
             "label": _("Mixed AdLib/MIDI mode"),
             "type": "bool",
             "default": False,
@@ -319,6 +411,7 @@ class scummvm(Runner):
         },
         {
             "option": "native-mt32",
+            "section": _("Audio"),
             "label": _("True Roland MT-32"),
             "type": "bool",
             "default": False,
@@ -328,6 +421,7 @@ class scummvm(Runner):
         },
         {
             "option": "enable-gs",
+            "section": _("Audio"),
             "label": _("Enable Roland GS"),
             "type": "bool",
             "default": False,
@@ -359,6 +453,7 @@ class scummvm(Runner):
         {
             "option": "debug-level",
             "type": "string",
+            "section": _("Debugging"),
             "label": _("Debug level"),
             "help": _("Sets debug verbosity level"),
             "advanced": True,
@@ -366,6 +461,7 @@ class scummvm(Runner):
         {
             "option": "debug-flags",
             "type": "string",
+            "section": _("Debugging"),
             "label": _("Debug flags"),
             "help": _("Enables engine specific debug flags"),
             "advanced": True,
@@ -376,14 +472,19 @@ class scummvm(Runner):
     def game_path(self):
         return self.game_config.get("path")
 
-    @property
-    def libs_dir(self):
+    def get_extra_libs(self):
+        """Scummvm runner ships additional, they may be removed in a future version."""
+        base_runner_path = os.path.join(settings.RUNNER_DIR, "scummvm")
+        if not self.get_executable().startswith(base_runner_path):
+            return ""
         path = os.path.join(settings.RUNNER_DIR, "scummvm/lib")
         return path if system.path_exists(path) else ""
 
     def get_command(self):
-        return [
-            self.get_executable(),
+        command = super().get_command()
+        if "flatpak" in command[0]:
+            return command
+        return command + [
             "--extrapath=%s" % self.get_scummvm_data_dir(),
             "--themepath=%s" % self.get_scummvm_data_dir(),
         ]
@@ -399,9 +500,9 @@ class scummvm(Runner):
 
     def get_run_data(self):
         env = self.get_env()
-        env["LD_LIBRARY_PATH"] = os.pathsep.join(filter(None, [
-            self.libs_dir,
-            env.get("LD_LIBRARY_PATH")]))
+        extra_libs = self.get_extra_libs()
+        if extra_libs:
+            env["LD_LIBRARY_PATH"] = ":".join([p for p in [extra_libs, env.get("LD_LIBRARY_PATH")] if p])
         return {"env": env, "command": self.get_command()}
 
     def inject_runner_option(self, command, key, cmdline, cmdline_empty=None):
@@ -423,12 +524,18 @@ class scummvm(Runner):
         for arg in split_arguments(args):
             command.append(arg)
         command.append(self.game_config.get("game_id"))
-        return {"command": command, "ld_library_path": self.libs_dir}
+        output = {"command": command}
+        extra_libs = self.get_extra_libs()
+        if extra_libs:
+            output["ld_library_path"] = extra_libs
+        return output
 
     def get_game_list(self):
         """Return the entire list of games supported by ScummVM."""
-        with subprocess.Popen([self.get_executable(), "--list-games"],
-                              stdout=subprocess.PIPE, encoding="utf-8", universal_newlines=True) as scummvm_process:
+        with subprocess.Popen(self.get_command() + ["--list-games"],
+                              stdout=subprocess.PIPE,
+                              encoding="utf-8",
+                              universal_newlines=True) as scummvm_process:
             scumm_output = scummvm_process.communicate()[0]
             game_list = str.split(scumm_output, "\n")
         game_array = []
