@@ -12,6 +12,7 @@ from lutris.util.log import logger
 class vice(Runner):
     description = _("Commodore Emulator")
     human_name = _("Vice")
+    # flatpak_id = "net.sf.VICE"  # needs adjustments
     platforms = [
         _("Commodore 64"),
         _("Commodore 128"),
@@ -54,34 +55,38 @@ class vice(Runner):
         {
             "option": "fullscreen",
             "type": "bool",
+            "section": _("Graphics"),
             "label": _("Fullscreen"),
             "default": False,
         },
         {
             "option": "double",
             "type": "bool",
+            "section": _("Graphics"),
             "label": _("Scale up display by 2"),
             "default": True,
         },
         {
             "option": "aspect_ratio",
             "type": "bool",
+            "section": _("Graphics"),
             "label": _("Preserve aspect ratio"),
             "default": True,
         },
+        {
+            "option": "renderer",
+            "type": "choice",
+            "section": _("Graphics"),
+            "label": _("Graphics renderer"),
+            "choices": [("OpenGL", "opengl"), (_("Software"), "software")],
+            "default": "opengl",
+        },
         {
             "option": "drivesound",
             "type": "bool",
             "label": _("Enable sound emulation of disk drives"),
             "default": False,
         },
-        {
-            "option": "renderer",
-            "type": "choice",
-            "label": _("Graphics renderer"),
-            "choices": [("OpenGL", "opengl"), (_("Software"), "software")],
-            "default": "opengl",
-        },
         {
             "option": "machine",
             "type": "choice",
@@ -116,7 +121,7 @@ class vice(Runner):
             raise ValueError("Invalid machine '%s'" % machine) from ex
         return os.path.join(settings.RUNNER_DIR, "vice/bin/%s" % executable)
 
-    def install(self, version=None, downloader=None, callback=None):
+    def install(self, install_ui_delegate, version=None, callback=None):
 
         def on_runner_installed(*args):
             config_path = system.create_folder("~/.vice")
@@ -130,7 +135,7 @@ class vice(Runner):
             if callback:
                 callback()
 
-        super().install(version, downloader, on_runner_installed)
+        super().install(install_ui_delegate, version, on_runner_installed)
 
     def get_roms_path(self, machine=None):
         if not machine:
