@@ -5,6 +5,7 @@ import shlex
 from gettext import gettext as _
 
 from lutris import runtime, settings
+from lutris.exceptions import EsyncLimitError, FsyncUnsupportedError
 from lutris.gui.dialogs import FileDialog
 from lutris.runners.commands.wine import (  # noqa: F401 pylint: disable=unused-import
     create_prefix, delete_registry_key, eject_disc, install_cab_component, open_wine_terminal, set_regedit,
@@ -13,26 +14,146 @@ from lutris.runners.commands.wine import (  # noqa: F401 pylint: disable=unused-
 from lutris.runners.runner import Runner
 from lutris.util import system
 from lutris.util.display import DISPLAY_MANAGER, get_default_dpi
-from lutris.util.graphics.vkquery import is_vulkan_supported
-from lutris.util.jobs import thread_safe_call
+from lutris.util.graphics import drivers, vkquery
+from lutris.util.linux import LINUX_SYSTEM
 from lutris.util.log import logger
-from lutris.util.strings import parse_version, split_arguments
+from lutris.util.steam.config import get_steam_dir
+from lutris.util.strings import split_arguments
 from lutris.util.wine.d3d_extras import D3DExtrasManager
 from lutris.util.wine.dgvoodoo2 import dgvoodoo2Manager
-from lutris.util.wine.dxvk import DXVKManager
+from lutris.util.wine.dxvk import REQUIRED_VULKAN_API_VERSION, DXVKManager
 from lutris.util.wine.dxvk_nvapi import DXVKNVAPIManager
 from lutris.util.wine.extract_icon import PEFILE_AVAILABLE, ExtractIcon
 from lutris.util.wine.prefix import DEFAULT_DLL_OVERRIDES, WinePrefixManager, find_prefix
 from lutris.util.wine.vkd3d import VKD3DManager
 from lutris.util.wine.wine import (
-    POL_PATH, WINE_DIR, WINE_PATHS, detect_arch, display_vulkan_error, esync_display_limit_warning,
-    esync_display_version_warning, fsync_display_support_warning, fsync_display_version_warning, get_default_version,
-    get_overrides_env, get_proton_paths, get_real_executable, get_wine_version, get_wine_versions, is_esync_limit_set,
-    is_fsync_supported, is_gstreamer_build, is_version_esync, is_version_fsync
+    WINE_DEFAULT_ARCH, WINE_DIR, WINE_PATHS, detect_arch, get_default_version, get_installed_wine_versions,
+    get_overrides_env, get_proton_paths, get_real_executable, get_system_wine_version, is_esync_limit_set,
+    is_fsync_supported, is_gstreamer_build
 )
 
-DEFAULT_WINE_PREFIX = "~/.wine"
-MIN_SAFE_VERSION = "7.0"  # Wine installers must run with at least this version
+
+def _get_prefix_warning(config, _option_key):
+    if config.get("prefix"):
+        return None
+
+    exe = config.get("exe")
+    if exe and find_prefix(exe):
+        return None
+
+    return _("<b>Warning</b> Some Wine configuration options cannot be applied, if no prefix can be found.")
+
+
+def _get_dxvk_warning(config, option_key):
+    if drivers.is_outdated():
+        driver_info = drivers.get_nvidia_driver_info()
+        return _("<b>Warning</b> Your NVIDIA driver is outdated.\n"
+                 "You are currently running driver %s which does not "
+                 "fully support all features for Vulkan and DXVK games.\n"
+                 "Please upgrade your driver as described in our "
+                 "<a href='%s'>installation guide</a>"
+                 ) % (
+            driver_info["nvrm"]["version"],
+            settings.DRIVER_HOWTO_URL,
+        )
+
+    return None
+
+
+def _get_dxvk_error(config, option_key):
+    """Checks that required libraries are installed on the system"""
+    missing_arch = LINUX_SYSTEM.get_missing_lib_arch("VULKAN")
+    if missing_arch:
+        arches = ", ".join(missing_arch)
+        return _("<b>Error</b> Missing Vulkan libraries\n"
+                 "Lutris was unable to detect Vulkan support for "
+                 "the %s architecture.\n"
+                 "This will prevent many games and programs from working.\n"
+                 "To install it, please use the following guide: "
+                 "<a href='%s'>Installing Graphics Drivers</a>"
+                 ) % (arches, settings.DRIVER_HOWTO_URL)
+
+    if not LINUX_SYSTEM.is_vulkan_supported():
+        return _("<b>Error</b> Vulkan is not installed or is not supported by your system, so DXVK is not available.\n"
+                 "If you have compatible hardware, please follow "
+                 "the installation procedures as described in\n"
+                 "<a href='https://github.com/lutris/docs/blob/master/HowToDXVK.md'>"
+                 "How-to:-DXVK (https://github.com/lutris/docs/blob/master/HowToDXVK.md)</a>"
+                 )
+
+    return None
+
+
+def _get_simple_vulkan_support_error(config, option_key, feature):
+    if not LINUX_SYSTEM.is_vulkan_supported():
+        return _("<b>Error</b> Vulkan is not installed or is not supported by your system, "
+                 "so %s is not available.") % feature
+    return None
+
+
+def _get_dxvk_version_warning(config, _option_key):
+    if config.get("dxvk") and LINUX_SYSTEM.is_vulkan_supported():
+        version = config.get("dxvk_version")
+        if version and not version.startswith("v1."):
+            library_api_version = vkquery.get_vulkan_api_version()
+            if library_api_version and library_api_version < REQUIRED_VULKAN_API_VERSION:
+                return _("<b>Warning</b> Lutris has detected that Vulkan API version %s is installed, "
+                         "but to use the latest DXVK version, %s is required."
+                         ) % (
+                    vkquery.format_version(library_api_version),
+                    vkquery.format_version(REQUIRED_VULKAN_API_VERSION)
+                )
+
+            devices = vkquery.get_device_info()
+
+            if devices and devices[0].api_version < REQUIRED_VULKAN_API_VERSION:
+                return _(
+                    "<b>Warning</b> Lutris has detected that the best device available ('%s') supports Vulkan API %s, "
+                    "but to use the latest DXVK version, %s is required."
+                ) % (
+                    devices[0].name,
+                    vkquery.format_version(devices[0].api_version),
+                    vkquery.format_version(REQUIRED_VULKAN_API_VERSION)
+                )
+
+    return None
+
+
+def _get_path_for_version(config, version=None):
+    """Return the absolute path of a wine executable for a given version,
+    or the configured version if you don't ask for a version."""
+    if not version:
+        version = config["version"]
+
+    if version in WINE_PATHS:
+        return system.find_executable(WINE_PATHS[version])
+    if "Proton" in version:
+        for proton_path in get_proton_paths():
+            if os.path.isfile(os.path.join(proton_path, version, "dist/bin/wine")):
+                return os.path.join(proton_path, version, "dist/bin/wine")
+            if os.path.isfile(os.path.join(proton_path, version, "files/bin/wine")):
+                return os.path.join(proton_path, version, "files/bin/wine")
+    if version == "custom":
+        return config.get("custom_wine_path", "")
+    return os.path.join(WINE_DIR, version, "bin/wine")
+
+
+def _get_esync_warning(config, _option_key):
+    if config.get("esync"):
+        limits_set = is_esync_limit_set()
+        if not limits_set:
+            return _("<b>Warning</b> Your limits are not set correctly. Please increase them as described here:\n"
+                     "<a href='https://github.com/lutris/docs/blob/master/HowToEsync.md'>"
+                     "How-to-Esync (https://github.com/lutris/docs/blob/master/HowToEsync.md)</a>")
+    return ""
+
+
+def _get_fsync_warning(config, _option_key):
+    if config.get("fsync"):
+        fsync_supported = is_fsync_supported()
+        if not fsync_supported:
+            return _("<b>Warning</b> Your kernel is not patched for fsync.")
+        return ""
 
 
 class wine(Runner):
@@ -70,6 +191,7 @@ class wine(Runner):
             "option": "prefix",
             "type": "directory_chooser",
             "label": _("Wine prefix"),
+            "warning": _get_prefix_warning,
             "help": _(
                 'The prefix used by Wine.\n'
                 "It's a directory containing a set of files and "
@@ -105,8 +227,11 @@ class wine(Runner):
         "wineboot.exe",
     )
 
-    def __init__(self, config=None):  # noqa: C901
+    def __init__(self, config=None, prefix=None, working_dir=None, wine_arch=None):  # noqa: C901
         super().__init__(config)
+        self._prefix = prefix
+        self._working_dir = working_dir
+        self._wine_arch = wine_arch
         self.dll_overrides = DEFAULT_DLL_OVERRIDES.copy()  # we'll modify this, so we better copy it
 
         def get_wine_version_choices():
@@ -117,53 +242,16 @@ class wine(Runner):
                 "wine-development": _("Wine Development ({})"),
                 "system": _("System ({})"),
             }
-            versions = get_wine_versions()
+            versions = get_installed_wine_versions()
             for version in versions:
                 if version in labels:
-                    version_number = get_wine_version(WINE_PATHS[version])
+                    version_number = get_system_wine_version(WINE_PATHS[version])
                     label = labels[version].format(version_number)
                 else:
                     label = version
                 version_choices.append((label, version))
             return version_choices
 
-        def esync_limit_callback(widget, option, config):
-            limits_set = is_esync_limit_set()
-            wine_path = self.get_path_for_version(config["version"])
-            wine_ver = is_version_esync(wine_path)
-            response = True
-
-            if not wine_ver:
-                response = thread_safe_call(esync_display_version_warning)
-
-            if not limits_set:
-                thread_safe_call(esync_display_limit_warning)
-                response = False
-
-            return widget, option, response
-
-        def fsync_support_callback(widget, option, config):
-            fsync_supported = is_fsync_supported()
-            wine_path = self.get_path_for_version(config["version"])
-            wine_ver = is_version_fsync(wine_path)
-            response = True
-
-            if not wine_ver:
-                response = thread_safe_call(fsync_display_version_warning)
-
-            if not fsync_supported:
-                thread_safe_call(fsync_display_support_warning)
-                response = False
-
-            return widget, option, response
-
-        def dxvk_vulkan_callback(widget, option, config):
-            response = True
-            if not is_vulkan_supported():
-                if not thread_safe_call(display_vulkan_error):
-                    response = False
-            return widget, option, response
-
         self.runner_options = [
             {
                 "option": "version",
@@ -195,11 +283,12 @@ class wine(Runner):
             },
             {
                 "option": "dxvk",
+                "section": _("Graphics"),
                 "label": _("Enable DXVK"),
-                "type": "extended_bool",
-                "callback": dxvk_vulkan_callback,
-                "callback_on": True,
+                "type": "bool",
                 "default": True,
+                "warning": _get_dxvk_warning,
+                "error": _get_dxvk_error,
                 "active": True,
                 "help": _(
                     "Use DXVK to "
@@ -208,19 +297,22 @@ class wine(Runner):
             },
             {
                 "option": "dxvk_version",
+                "section": _("Graphics"),
                 "label": _("DXVK version"),
                 "advanced": True,
                 "type": "choice_with_entry",
+                "condition": LINUX_SYSTEM.is_vulkan_supported(),
                 "choices": DXVKManager().version_choices,
                 "default": DXVKManager().version,
+                "warning": _get_dxvk_version_warning,
             },
 
             {
                 "option": "vkd3d",
+                "section": _("Graphics"),
                 "label": _("Enable VKD3D"),
-                "type": "extended_bool",
-                "callback": dxvk_vulkan_callback,
-                "callback_on": True,
+                "type": "bool",
+                "error": lambda c, k: _get_simple_vulkan_support_error(c, k, _("VKD3D")),
                 "default": True,
                 "active": True,
                 "help": _(
@@ -229,14 +321,17 @@ class wine(Runner):
             },
             {
                 "option": "vkd3d_version",
+                "section": _("Graphics"),
                 "label": _("VKD3D version"),
                 "advanced": True,
                 "type": "choice_with_entry",
+                "condition": LINUX_SYSTEM.is_vulkan_supported(),
                 "choices": VKD3DManager().version_choices,
                 "default": VKD3DManager().version,
             },
             {
                 "option": "d3d_extras",
+                "section": _("Graphics"),
                 "label": _("Enable D3D Extras"),
                 "type": "bool",
                 "default": True,
@@ -248,6 +343,7 @@ class wine(Runner):
             },
             {
                 "option": "d3d_extras_version",
+                "section": _("Graphics"),
                 "label": _("D3D Extras version"),
                 "advanced": True,
                 "type": "choice_with_entry",
@@ -256,8 +352,10 @@ class wine(Runner):
             },
             {
                 "option": "dxvk_nvapi",
+                "section": _("Graphics"),
                 "label": _("Enable DXVK-NVAPI / DLSS"),
                 "type": "bool",
+                "error": lambda c, k: _get_simple_vulkan_support_error(c, k, _("DXVK-NVAPI / DLSS")),
                 "default": True,
                 "advanced": True,
                 "help": _(
@@ -266,6 +364,7 @@ class wine(Runner):
             },
             {
                 "option": "dxvk_nvapi_version",
+                "section": _("Graphics"),
                 "label": _("DXVK NVAPI version"),
                 "advanced": True,
                 "type": "choice_with_entry",
@@ -274,6 +373,7 @@ class wine(Runner):
             },
             {
                 "option": "dgvoodoo2",
+                "section": _("Graphics"),
                 "label": _("Enable dgvoodoo2"),
                 "type": "bool",
                 "default": False,
@@ -286,6 +386,7 @@ class wine(Runner):
             },
             {
                 "option": "dgvoodoo2_version",
+                "section": _("Graphics"),
                 "label": _("dgvoodoo2 version"),
                 "advanced": True,
                 "type": "choice_with_entry",
@@ -295,9 +396,8 @@ class wine(Runner):
             {
                 "option": "esync",
                 "label": _("Enable Esync"),
-                "type": "extended_bool",
-                "callback": esync_limit_callback,
-                "callback_on": True,
+                "type": "bool",
+                "warning": _get_esync_warning,
                 "active": True,
                 "default": True,
                 "help": _(
@@ -309,10 +409,9 @@ class wine(Runner):
             {
                 "option": "fsync",
                 "label": _("Enable Fsync"),
-                "type": "extended_bool",
+                "type": "bool",
                 "default": is_fsync_supported(),
-                "callback": fsync_support_callback,
-                "callback_on": True,
+                "warning": _get_fsync_warning,
                 "active": True,
                 "help": _(
                     "Enable futex-based synchronization (fsync). "
@@ -354,6 +453,7 @@ class wine(Runner):
             },
             {
                 "option": "Desktop",
+                "section": _("Virtual Desktop"),
                 "label": _("Windowed (virtual desktop)"),
                 "type": "bool",
                 "default": False,
@@ -365,6 +465,7 @@ class wine(Runner):
             },
             {
                 "option": "WineDesktop",
+                "section": _("Virtual Desktop"),
                 "label": _("Virtual desktop resolution"),
                 "type": "choice_with_entry",
                 "choices": DISPLAY_MANAGER.get_resolutions,
@@ -372,6 +473,7 @@ class wine(Runner):
             },
             {
                 "option": "Dpi",
+                "section": _("DPI"),
                 "label": _("Enable DPI Scaling"),
                 "type": "bool",
                 "default": False,
@@ -382,6 +484,7 @@ class wine(Runner):
             },
             {
                 "option": "ExplicitDpi",
+                "section": _("DPI"),
                 "label": _("DPI"),
                 "type": "string",
                 "help": _(
@@ -461,24 +564,26 @@ class wine(Runner):
                 "advanced": True,
                 "default": False,
                 "help":
-                _("Automatically disables one of Wine's detected joypad "
-                  "to avoid having 2 controllers detected"),
+                    _("Automatically disables one of Wine's detected joypad "
+                      "to avoid having 2 controllers detected"),
             },
             {
                 "option": "sandbox",
                 "type": "bool",
+                "section": _("Sandbox"),
                 "label": _("Create a sandbox for Wine folders"),
                 "default": True,
                 "advanced": True,
                 "help": _(
                     "Do not use $HOME for desktop integration folders.\n"
-                    "By default, it use the directories in the confined "
+                    "By default, it will use the directories in the confined "
                     "Windows environment."
                 ),
             },
             {
                 "option": "sandbox_dir",
                 "type": "directory_chooser",
+                "section": _("Sandbox"),
                 "label": _("Sandbox directory"),
                 "help": _("Custom directory for desktop integration folders."),
                 "advanced": True,
@@ -490,25 +595,24 @@ class wine(Runner):
         """Return the contexual menu entries for wine"""
         return [
             ("wineexec", _("Run EXE inside Wine prefix"), self.run_wineexec),
-            ("winecfg", _("Wine configuration"), self.run_winecfg),
             ("wineshell", _("Open Bash terminal"), self.run_wine_terminal),
             ("wineconsole", _("Open Wine console"), self.run_wineconsole),
+            (None, "-", None),
+            ("winecfg", _("Wine configuration"), self.run_winecfg),
             ("wine-regedit", _("Wine registry"), self.run_regedit),
-            ("winetricks", _("Winetricks"), self.run_winetricks),
             ("winecpl", _("Wine Control Panel"), self.run_winecpl),
+            (None, "-", None),
+            ("winetricks", _("Winetricks"), self.run_winetricks),
         ]
 
     @property
     def prefix_path(self):
-        """Return the absolute path of the Wine prefix"""
-        _prefix_path = self.game_config.get("prefix") \
-            or os.environ.get("WINEPREFIX")
+        """Return the absolute path of the Wine prefix. Falls back to default WINE prefix."""
+        _prefix_path = self._prefix or self.game_config.get("prefix") or os.environ.get("WINEPREFIX")
         if not _prefix_path and self.game_config.get("exe"):
             # Find prefix from game if we have one
             _prefix_path = find_prefix(self.game_exe)
-        if not _prefix_path:
-            _prefix_path = DEFAULT_WINE_PREFIX
-        return os.path.expanduser(_prefix_path)
+        return _prefix_path
 
     @property
     def game_exe(self):
@@ -528,9 +632,9 @@ class wine(Runner):
     @property
     def working_dir(self):
         """Return the working directory to use when running the game."""
-        option = self.game_config.get("working_dir")
-        if option:
-            return option
+        _working_dir = self._working_dir or self.game_config.get("working_dir")
+        if _working_dir:
+            return _working_dir
         if self.game_exe:
             game_dir = os.path.dirname(self.game_exe)
             if os.path.isdir(game_dir):
@@ -547,41 +651,66 @@ class wine(Runner):
         """Return the wine architecture.
 
         Get it from the config or detect it from the prefix"""
-        arch = self.game_config.get("arch") or "auto"
+        arch = self._wine_arch or self.game_config.get("arch") or "auto"
         if arch not in ("win32", "win64"):
-            arch = detect_arch(self.prefix_path, self.get_executable())
+            prefix_path = self.prefix_path
+            if prefix_path:
+                arch = detect_arch(self.prefix_path, self.get_executable())
+            else:
+                arch = WINE_DEFAULT_ARCH
         return arch
 
-    def get_version(self, use_default=True):
+    def get_runner_version(self, version=None):
+        if not version:
+            version = self.read_version_from_config(use_default=False)
+
+        if version in WINE_PATHS:
+            return {"version": version}
+
+        return super().get_runner_version(version)
+
+    def read_version_from_config(self, use_default=True):
         """Return the Wine version to use. use_default can be set to false to
         force the installation of a specific wine version"""
-        runner_version = self.runner_config.get("version")
-        if runner_version:
-            return runner_version
+
+        # We must use the config levels to avoid getting a default if the setting
+        # is not set; we'll fall back to get_default_version() or None rather.
+
+        for level in [self.config.game_level, self.config.runner_level]:
+            if "wine" in level:
+                runner_version = level["wine"].get("version")
+                if runner_version:
+                    return runner_version
         if use_default:
             return get_default_version()
 
     def get_path_for_version(self, version):
         """Return the absolute path of a wine executable for a given version"""
-        if version in WINE_PATHS:
-            return system.find_executable(WINE_PATHS[version])
-        if "Proton" in version:
-            for proton_path in get_proton_paths():
-                if os.path.isfile(os.path.join(proton_path, version, "dist/bin/wine")):
-                    return os.path.join(proton_path, version, "dist/bin/wine")
-        if version.startswith("PlayOnLinux"):
-            version, arch = version.split()[1].rsplit("-", 1)
-            return os.path.join(POL_PATH, "wine", "linux-" + arch, version, "bin/wine")
-        if version == "custom":
-            return self.runner_config.get("custom_wine_path", "")
-        return os.path.join(WINE_DIR, version, "bin/wine")
+        return _get_path_for_version(self.runner_config, version)
+
+    def resolve_config_path(self, path, relative_to=None):
+        # Resolve paths with tolerance for Windows-isms;
+        # first try to fix mismatched casing, and then if that
+        # finds no file or directory, try again after swapping in
+        # slashes for backslashes.
+
+        resolved = super().resolve_config_path(path, relative_to)
+        resolved = system.fix_path_case(resolved)
+
+        if not os.path.exists(resolved) and '\\' in path:
+            fixed = path.replace('\\', '/')
+            fixed_resolved = super().resolve_config_path(fixed, relative_to)
+            fixed_resolved = system.fix_path_case(fixed_resolved)
+            return fixed_resolved
+
+        return resolved
 
     def get_executable(self, version=None, fallback=True):
         """Return the path to the Wine executable.
         A specific version can be specified if needed.
         """
         if version is None:
-            version = self.get_version()
+            version = self.read_version_from_config()
         if not version:
             return
 
@@ -604,22 +733,13 @@ class wine(Runner):
                     # which one to get the correct LutrisConfig object.
             return wine_path
 
-    def is_installed(self, version=None, fallback=True, min_version=None):
+    def is_installed(self, version=None, fallback=True):
         """Check if Wine is installed.
         If no version is passed, checks if any version of wine is available
         """
         if version:
             return system.path_exists(self.get_executable(version, fallback))
-
-        wine_versions = get_wine_versions()
-        if min_version:
-            min_version_list, _, _ = parse_version(min_version)
-            for wine_version in wine_versions:
-                version_list, _, _ = parse_version(wine_version)
-                if version_list > min_version_list:
-                    return True
-            logger.warning("Wine %s or higher not found", min_version)
-        return bool(wine_versions)
+        return bool(get_installed_wine_versions())
 
     @classmethod
     def msi_exec(
@@ -688,11 +808,13 @@ class wine(Runner):
 
     def run_wine_terminal(self, *args):
         terminal = self.system_config.get("terminal_app")
+        system_winetricks = self.runner_config.get("system_winetricks")
         open_wine_terminal(
             terminal=terminal,
             wine_path=self.get_executable(),
             prefix=self.prefix_path,
-            env=self.get_env(os_env=True)
+            env=self.get_env(os_env=True),
+            system_winetricks=system_winetricks,
         )
 
     def run_winetricks(self, *args):
@@ -770,72 +892,63 @@ class wine(Runner):
             explicit_dpi = self.runner_config.get("ExplicitDpi")
             if explicit_dpi == "auto":
                 explicit_dpi = None
-            try:
-                explicit_dpi = int(explicit_dpi)
-            except:
-                explicit_dpi = None
+            else:
+                try:
+                    explicit_dpi = int(explicit_dpi)
+                except:
+                    explicit_dpi = None
             return explicit_dpi or get_default_dpi()
 
         return None
 
-    def setup_dlls(self, manager_class, enable, version):
-        """Enable or disable DLLs"""
-        dll_manager = manager_class(
-            self.prefix_path,
-            arch=self.wine_arch,
-            version=version,
-        )
-
-        # manual version only sets the dlls to native
-        manager_version = dll_manager.version
-        if not manager_version or manager_version.lower() != "manual":
-            if enable:
-                dll_manager.enable()
-            else:
-                dll_manager.disable()
-
-        if enable:
-            for dll in dll_manager.managed_dlls:
-                # We have to make sure that the dll exists before setting it to native
-                if dll_manager.dll_exists(dll):
-                    self.dll_overrides[dll] = "n"
-
     def prelaunch(self):
-        if not system.path_exists(os.path.join(self.prefix_path, "user.reg")):
-            logger.warning("No valid prefix detected in %s, creating one...", self.prefix_path)
-            create_prefix(self.prefix_path, wine_path=self.get_executable(), arch=self.wine_arch, runner=self)
+        prefix_path = self.prefix_path
+        if prefix_path:
+            if not system.path_exists(os.path.join(prefix_path, "user.reg")):
+                logger.warning("No valid prefix detected in %s, creating one...", prefix_path)
+                create_prefix(prefix_path, wine_path=self.get_executable(), arch=self.wine_arch, runner=self)
 
-        prefix_manager = WinePrefixManager(self.prefix_path)
-        if self.runner_config.get("autoconf_joypad", False):
-            prefix_manager.configure_joypads()
-        self.sandbox(prefix_manager)
-        self.set_regedit_keys()
+            prefix_manager = WinePrefixManager(prefix_path)
+            if self.runner_config.get("autoconf_joypad", False):
+                prefix_manager.configure_joypads()
+            prefix_manager.create_user_symlinks()
+            self.sandbox(prefix_manager)
+            self.set_regedit_keys()
 
-        self.setup_dlls(
-            DXVKManager,
-            bool(self.runner_config.get("dxvk")),
-            self.runner_config.get("dxvk_version")
-        )
-        self.setup_dlls(
-            VKD3DManager,
-            bool(self.runner_config.get("vkd3d")),
-            self.runner_config.get("vkd3d_version")
-        )
-        self.setup_dlls(
-            DXVKNVAPIManager,
-            bool(self.runner_config.get("dxvk_nvapi")),
-            self.runner_config.get("dxvk_nvapi_version")
-        )
-        self.setup_dlls(
-            D3DExtrasManager,
-            bool(self.runner_config.get("d3d_extras")),
-            self.runner_config.get("d3d_extras_version")
-        )
-        self.setup_dlls(
-            dgvoodoo2Manager,
-            bool(self.runner_config.get("dgvoodoo2")),
-            self.runner_config.get("dgvoodoo2_version")
-        )
+            for manager, enabled in self.get_dll_managers().items():
+                manager.setup(enabled)
+
+    def get_dll_managers(self, enabled_only=False):
+        """Returns the DLL managers in a dict; the keys are the managers themselves,
+        and the values are the enabled flags for them. If 'enabled_only' is true,
+        only enabled managers are returned, so disabled managers are not created."""
+        manager_classes = [
+            (DXVKManager, "dxvk", "dxvk_version"),
+            (VKD3DManager, "vkd3d", "vkd3d_version"),
+            (DXVKNVAPIManager, "dxvk_nvapi", "dxvk_nvapi_version"),
+            (D3DExtrasManager, "d3d_extras", "d3d_extras_version"),
+            (dgvoodoo2Manager, "dgvoodoo2", "dgvoodoo2_version")
+        ]
+
+        managers = {}
+        for manager_class, enabled_option, version_option in manager_classes:
+            enabled = bool(self.runner_config.get(enabled_option))
+            version = self.runner_config.get(version_option)
+            if enabled or not enabled_only:
+                manager = manager_class(
+                    self.prefix_path,
+                    arch=self.wine_arch,
+                    version=version
+                )
+
+                if not manager.can_enable():
+                    enabled = False
+                    if enabled_only:
+                        continue
+
+                managers[manager] = enabled
+
+        return managers
 
     def get_dll_overrides(self):
         """Return the DLLs overriden at runtime"""
@@ -861,11 +974,11 @@ class wine(Runner):
             env["DXVK_LOG_LEVEL"] = "none"
         env["WINEARCH"] = self.wine_arch
         env["WINE"] = self.get_executable()
-        env["WINE_MONO_CACHE_DIR"] = os.path.join(WINE_DIR, self.get_version(), "mono")
-        env["WINE_GECKO_CACHE_DIR"] = os.path.join(WINE_DIR, self.get_version(), "gecko")
+        env["WINE_MONO_CACHE_DIR"] = os.path.join(WINE_DIR, self.read_version_from_config(), "mono")
+        env["WINE_GECKO_CACHE_DIR"] = os.path.join(WINE_DIR, self.read_version_from_config(), "gecko")
         if is_gstreamer_build(self.get_executable()):
-            path_64 = os.path.join(WINE_DIR, self.get_version(), "lib64/gstreamer-1.0/")
-            path_32 = os.path.join(WINE_DIR, self.get_version(), "lib/gstreamer-1.0/")
+            path_64 = os.path.join(WINE_DIR, self.read_version_from_config(), "lib64/gstreamer-1.0/")
+            path_32 = os.path.join(WINE_DIR, self.read_version_from_config(), "lib/gstreamer-1.0/")
             if os.path.exists(path_64) or os.path.exists(path_32):
                 env["GST_PLUGIN_SYSTEM_PATH_1_0"] = path_64 + ":" + path_32
         if self.prefix_path:
@@ -890,10 +1003,25 @@ class wine(Runner):
         if self.runner_config.get("eac"):
             env["PROTON_EAC_RUNTIME"] = os.path.join(settings.RUNTIME_DIR, "eac_runtime")
 
+        for dll_manager in self.get_dll_managers(enabled_only=True):
+            self.dll_overrides.update(dll_manager.get_enabling_dll_overrides())
+
         overrides = self.get_dll_overrides()
         if overrides:
             self.dll_overrides.update(overrides)
+
         env["WINEDLLOVERRIDES"] = get_overrides_env(self.dll_overrides)
+
+        # Proton support
+        if "Proton" in self.read_version_from_config():
+            steam_dir = get_steam_dir()
+            if steam_dir:  # May be None for example if Proton-GE is used but Steam is not installed
+                env["STEAM_COMPAT_CLIENT_INSTALL_PATH"] = steam_dir
+            env["STEAM_COMPAT_DATA_PATH"] = self.prefix_path
+            env["STEAM_COMPAT_APP_ID"] = '0'
+            env["SteamAppId"] = '0'
+            if "SteamGameId" not in env:
+                env["SteamGameId"] = "lutris-game"
         return env
 
     def get_runtime_env(self):
@@ -931,9 +1059,9 @@ class wine(Runner):
 
     def sandbox(self, wine_prefix):
         if self.runner_config.get("sandbox", True):
-            wine_prefix.desktop_integration(desktop_dir=self.runner_config.get("sandbox_dir"))
+            wine_prefix.enable_desktop_integration_sandbox(desktop_dir=self.runner_config.get("sandbox_dir"))
         else:
-            wine_prefix.desktop_integration(restore=True)
+            wine_prefix.restore_desktop_integration()
 
     def play(self):  # pylint: disable=too-many-return-statements # noqa: C901
         game_exe = self.game_exe
@@ -944,42 +1072,21 @@ class wine(Runner):
         if using_dxvk:
             # Set this to 1 to enable access to more RAM for 32bit applications
             launch_info["env"]["WINE_LARGE_ADDRESS_AWARE"] = "1"
-            if not is_vulkan_supported():
-                if not display_vulkan_error(on_launch=True):
-                    return {"error": "VULKAN_NOT_FOUND"}
 
         if not game_exe or not system.path_exists(game_exe):
             return {"error": "FILE_NOT_FOUND", "file": game_exe}
 
         if launch_info["env"].get("WINEESYNC") == "1":
             limit_set = is_esync_limit_set()
-            wine_ver = is_version_esync(self.get_executable())
 
-            if not limit_set and not wine_ver:
-                esync_display_version_warning(True)
-                esync_display_limit_warning()
-                return {"error": "ESYNC_LIMIT_NOT_SET"}
-            if not is_esync_limit_set():
-                esync_display_limit_warning()
-                return {"error": "ESYNC_LIMIT_NOT_SET"}
-            if not wine_ver:
-                if not esync_display_version_warning(True):
-                    return {"error": "NON_ESYNC_WINE_VERSION"}
+            if not limit_set:
+                raise EsyncLimitError(_("Your ESYNC limits are not set correctly."))
 
         if launch_info["env"].get("WINEFSYNC") == "1":
             fsync_supported = is_fsync_supported()
-            wine_ver = is_version_fsync(self.get_executable())
 
-            if not fsync_supported and not wine_ver:
-                fsync_display_version_warning(True)
-                fsync_display_support_warning()
-                return {"error": "FSYNC_NOT_SUPPORTED"}
             if not fsync_supported:
-                fsync_display_support_warning()
-                return {"error": "FSYNC_NOT_SUPPORTED"}
-            if not wine_ver:
-                if not fsync_display_version_warning(True):
-                    return {"error": "NON_FSYNC_WINE_VERSION"}
+                raise FsyncUnsupportedError(_("Your kernel is not patched for fsync."))
 
         command = [self.get_executable()]
 
@@ -1019,9 +1126,9 @@ class wine(Runner):
         # Relative path
         return path
 
-    def extract_icon_exe(self, game_slug):
+    def extract_icon(self, game_slug):
         """Extracts the 128*128 icon from EXE and saves it, if not resizes the biggest icon found.
-            returns true if an icon is saved, false if not"""
+        returns true if an icon is saved, false if not"""
         try:
             wantedsize = (128, 128)
             pathtoicon = settings.ICON_PATH + "/lutris_" + game_slug + ".png"
