@@ -3,8 +3,9 @@ import os
 
 from lutris import settings
 from lutris.services.amazon import AmazonService
-from lutris.services.battlenet import BattleNetService
+from lutris.services.battlenet import BNET_ENABLED, BattleNetService
 from lutris.services.dolphin import DolphinService
+from lutris.services.ea_app import EAAppService
 from lutris.services.egs import EpicGamesStoreService
 from lutris.services.flathub import FlathubService
 from lutris.services.gog import GOGService
@@ -13,6 +14,7 @@ from lutris.services.itchio import ItchIoService
 from lutris.services.lutris import LutrisService
 from lutris.services.mame import MAMEService
 from lutris.services.origin import OriginService
+from lutris.services.scummvm import SCUMMVM_CONFIG_FILE, ScummvmService
 from lutris.services.steam import SteamService
 from lutris.services.steamwindows import SteamWindowsService
 from lutris.services.ubisoft import UbisoftConnectService
@@ -21,7 +23,7 @@ from lutris.util import system
 from lutris.util.dolphin.cache_reader import DOLPHIN_GAME_CACHE_FILE
 from lutris.util.linux import LINUX_SYSTEM
 
-DEFAULT_SERVICES = ["lutris", "gog", "egs", "origin", "ubisoft", "steam"]
+DEFAULT_SERVICES = ["lutris", "gog", "egs", "ea_app", "ubisoft", "steam"]
 
 
 def get_services():
@@ -31,10 +33,15 @@ def get_services():
         "gog": GOGService,
         "humblebundle": HumbleBundleService,
         "egs": EpicGamesStoreService,
+        "itchio": ItchIoService,
         "origin": OriginService,
+        "ea_app": EAAppService,
         "ubisoft": UbisoftConnectService,
-        "amazon": AmazonService
+        "amazon": AmazonService,
+        "flathub": FlathubService
     }
+    if BNET_ENABLED:
+        _services["battlenet"] = BattleNetService
     if not LINUX_SYSTEM.is_flatpak:
         _services["xdg"] = XDGService
     if LINUX_SYSTEM.has_steam:
@@ -42,6 +49,8 @@ def get_services():
     _services["steamwindows"] = SteamWindowsService
     if system.path_exists(DOLPHIN_GAME_CACHE_FILE):
         _services["dolphin"] = DolphinService
+    if system.path_exists(SCUMMVM_CONFIG_FILE):
+        _services["scummvm"] = ScummvmService
     return _services
 
 
@@ -50,10 +59,8 @@ SERVICES = get_services()
 
 # Those services are not yet ready to be used
 WIP_SERVICES = {
-    "battlenet": BattleNetService,
-    "itchio": ItchIoService,
     "mame": MAMEService,
-    "flathub": FlathubService
+
 }
 
 if os.environ.get("LUTRIS_ENABLE_ALL_SERVICES"):
