@@ -5,6 +5,7 @@ from gettext import gettext as _
 from PIL import Image
 
 from lutris import settings
+from lutris.runners.dolphin import PLATFORMS
 from lutris.services.base import BaseService
 from lutris.services.service_game import ServiceGame
 from lutris.services.service_media import ServiceMedia
@@ -60,6 +61,20 @@ class DolphinService(BaseService):
         """Pull install location from installer"""
         return os.path.dirname(installer["script"]["game"]["main_file"])
 
+    def get_game_platforms(self, db_game):
+        if "details" in db_game:
+            details = json.loads(db_game.get("details"))
+            if details and details.get("platform"):
+                platform_value = details["platform"]
+                if platform_value.isdigit():
+                    platform_number = int(details["platform"])
+                    if 0 <= platform_number < len(PLATFORMS):
+                        platform = PLATFORMS[platform_number]
+                        return [platform]
+
+                return [platform_value]
+        return None
+
 
 class DolphinGame(ServiceGame):
     """Game for the Dolphin emulator"""
@@ -71,10 +86,11 @@ class DolphinGame(ServiceGame):
     @classmethod
     def new_from_cache(cls, cache_entry):
         """Create a service game from an entry from the Dolphin cache"""
+        name = cache_entry["internal_name"] or os.path.splitext(cache_entry["file_name"])[0]
         service_game = cls()
-        service_game.name = cache_entry["internal_name"]
+        service_game.name = name
         service_game.appid = str(cache_entry["game_id"])
-        service_game.slug = slugify(cache_entry["internal_name"])
+        service_game.slug = slugify(name)
         service_game.icon = service_game.get_banner(cache_entry)
 
         service_game.details = json.dumps({
@@ -91,7 +107,7 @@ class DolphinGame(ServiceGame):
 
     def get_banner(self, cache_entry):
         banner = DolphinBanner()
-        banner_path = banner.get_absolute_path(self.appid)
+        banner_path = banner.get_media_path(self.appid)
 
         if os.path.exists(banner_path):
             return banner_path
