@@ -17,7 +17,7 @@ from lutris.services.service_game import ServiceGame
 from lutris.services.service_media import ServiceMedia
 from lutris.util.log import logger
 from lutris.util.steam.appmanifest import AppManifest, get_appmanifests
-from lutris.util.steam.config import get_steam_library, get_steamapps_paths, get_user_steam_id
+from lutris.util.steam.config import get_active_steamid64, get_steam_library, get_steamapps_dirs
 from lutris.util.strings import slugify
 
 
@@ -26,6 +26,7 @@ class SteamBanner(ServiceMedia):
     size = (184, 69)
     dest_path = os.path.join(settings.CACHE_DIR, "steam/banners")
     file_pattern = "%s.jpg"
+    file_format = "jpeg"
     api_field = "appid"
     url_pattern = "http://cdn.akamai.steamstatic.com/steam/apps/%s/capsule_184x69.jpg"
 
@@ -35,6 +36,7 @@ class SteamCover(ServiceMedia):
     size = (200, 300)
     dest_path = os.path.join(settings.CACHE_DIR, "steam/covers")
     file_pattern = "%s.jpg"
+    file_format = "jpeg"
     api_field = "appid"
     url_pattern = "http://cdn.steamstatic.com/steam/apps/%s/library_600x900.jpg"
 
@@ -44,6 +46,7 @@ class SteamBannerLarge(ServiceMedia):
     size = (460, 215)
     dest_path = os.path.join(settings.CACHE_DIR, "steam/header")
     file_pattern = "%s.jpg"
+    file_format = "jpeg"
     api_field = "appid"
     url_pattern = "https://cdn.cloudflare.steamstatic.com/steam/apps/%s/header.jpg"
 
@@ -78,7 +81,6 @@ class SteamService(BaseService):
         "cover": SteamCover,
     }
     default_format = "banner"
-    is_loading = False
     runner = "steam"
     excluded_appids = [
         "221410",  # Steam for Linux
@@ -89,11 +91,7 @@ class SteamService(BaseService):
 
     def load(self):
         """Return importable Steam games"""
-        if self.is_loading:
-            logger.warning("Steam games are already loading")
-            return
-        self.is_loading = True
-        steamid = get_user_steam_id()
+        steamid = get_active_steamid64()
         if not steamid:
             logger.error("Unable to find SteamID from Steam config")
             return
@@ -106,10 +104,9 @@ class SteamService(BaseService):
             game = self.game_class.new_from_steam_game(steam_game)
             game.save()
         self.match_games()
-        self.is_loading = False
         return steam_games
 
-    def get_installer_files(self, installer, installer_file_id):
+    def get_installer_files(self, installer, installer_file_id, _selected_extras):
         steam_uri = "$STEAM:%s:."
         appid = str(installer.script["game"]["appid"])
         return [
@@ -151,7 +148,7 @@ class SteamService(BaseService):
 
     @property
     def steamapps_paths(self):
-        return get_steamapps_paths()
+        return get_steamapps_dirs()
 
     def add_installed_games(self):
         """Syncs installed Steam games with Lutris"""
@@ -196,8 +193,9 @@ class SteamService(BaseService):
             for game_id in game_ids:
                 steam_game = Game(game_id)
                 if not steam_game.playtime:
+                    # Unsafe to emit a signal from a worker thread!
                     steam_game.remove(no_signal=True)
-                    steam_game.delete()
+                    steam_game.delete(no_signal=True)
                     stats["deduped"] += 1
         logger.debug("%s Steam games deduplicated", stats["deduped"])
 
