@@ -1,6 +1,7 @@
 """Wine prefix management"""
 import os
 
+from lutris.settings import get_lutris_directory_settings, set_lutris_directory_settings
 from lutris.util import joypad, system
 from lutris.util.display import DISPLAY_MANAGER
 from lutris.util.log import logger
@@ -48,16 +49,19 @@ class WinePrefixManager:
             logger.warning("No path specified for Wine prefix")
         self.path = path
 
+    def get_user_dir(self, default_user=None):
+        user = default_user or os.getenv("USER") or "lutrisuser"
+        return os.path.join(self.path, "drive_c/users/", user)
+
     @property
     def user_dir(self):
         """Returns the directory that contains the current user's profile in the WINE prefix."""
-        user = os.getenv("USER") or 'lutrisuser'
-        return os.path.join(self.path, "drive_c/users/", user)
+        return self.get_user_dir()
 
     @property
     def appdata_dir(self):
         """Returns the app-data directory for the user; this depends on a registry key."""
-        user_dir = self.user_dir
+        user_dir = self.get_user_dir()
         folder = self.get_registry_key(
             self.hkcu_prefix + "/Software/Microsoft/Windows/CurrentVersion/Explorer/Shell Folders",
             "AppData",
@@ -77,11 +81,20 @@ class WinePrefixManager:
         for dll, value in DEFAULT_DLL_OVERRIDES.items():
             self.override_dll(dll, value)
         try:
-            self.desktop_integration()
+            self.enable_desktop_disintegration()
         except OSError as ex:
             logger.error("Failed to setup desktop integration, the prefix may not be valid.")
             logger.exception(ex)
 
+    def create_user_symlinks(self):
+        """Link together user profiles created by Wine and Proton"""
+        wine_user_dir = self.get_user_dir()
+        proton_user_dir = self.get_user_dir(default_user="steamuser")
+        if os.path.exists(wine_user_dir) and not os.path.exists(proton_user_dir):
+            os.symlink(wine_user_dir, proton_user_dir, target_is_directory=True)
+        elif os.path.exists(proton_user_dir) and not os.path.exists(wine_user_dir):
+            os.symlink(proton_user_dir, wine_user_dir, target_is_directory=True)
+
     def get_registry_path(self, key):
         """Matches registry keys to a registry file
 
@@ -141,61 +154,116 @@ class WinePrefixManager:
             desktop_folders.append(folder[folder.rfind("\\") + 1:])
         return desktop_folders or DEFAULT_DESKTOP_FOLDERS
 
-    def desktop_integration(self, desktop_dir=None, restore=False):  # noqa: C901
-        """Overwrite desktop integration"""
-        # pylint: disable=too-many-branches
-        # TODO: reduce complexity (18)
+    def enable_desktop_integration_sandbox(self, desktop_dir):
+        """Replace WINE desktop folders with links to a sandbox directory."""
         user_dir = self.user_dir
-        desktop_folders = self.get_desktop_folders()
         desktop_dir = os.path.expanduser(desktop_dir) if desktop_dir else user_dir
 
-        if system.path_exists(user_dir):
-            # Replace or restore desktop integration symlinks
+        if desktop_dir == user_dir:
+            self.enable_desktop_disintegration()
+            return
+
+        if system.path_exists(user_dir) and self._get_desktop_integration_assignment() != desktop_dir:
+            desktop_folders = self.get_desktop_folders()
+            for item in desktop_folders:
+                path = os.path.join(user_dir, item)
+                safe_path = path + ".winecfg"
+
+                self._remove_desktop_folder(path, safe_path)
+
+                try:
+                    src_path = os.path.join(desktop_dir, item)
+                except TypeError as ex:
+                    # There is supposedly a None value in there
+                    # The current code shouldn't allow that
+                    # Just raise a exception with the values
+                    raise RuntimeError("Missing value desktop_dir=%s or item=%s" % (desktop_dir, item)) from ex
+
+                os.makedirs(src_path, exist_ok=True)
+                os.symlink(src_path, path)
+
+            self._set_desktop_integration_assignment(desktop_dir)
+
+    def enable_desktop_disintegration(self):
+        """Replace the desktop integration links with proper folders."""
+        user_dir = self.user_dir
+
+        if system.path_exists(user_dir) and self._get_desktop_integration_assignment() != user_dir:
+            desktop_folders = self.get_desktop_folders()
+            for item in desktop_folders:
+                path = os.path.join(user_dir, item)
+                safe_path = path + ".winecfg"
+
+                # Disintegration means the desktop folders in WINE are
+                # actual directories not links, and that's all we want.
+                if not os.path.islink(path):
+                    continue
+
+                self._remove_desktop_folder(path, safe_path)
+
+                # We prefer to restore the previously saved directory.
+                if os.path.isdir(safe_path):
+                    os.rename(safe_path, path)
+                else:
+                    os.makedirs(path, exist_ok=True)
+
+            self._set_desktop_integration_assignment(user_dir)
+
+    def restore_desktop_integration(self):
+        """Replace WINE's desktop folders with links to the corresponding
+        folders in your home directory."""
+        user_dir = self.user_dir
+        home_dir = os.path.expanduser("~")
+
+        if system.path_exists(user_dir) and self._get_desktop_integration_assignment() != home_dir:
+            desktop_folders = self.get_desktop_folders()
             for i, item in enumerate(desktop_folders):
                 path = os.path.join(user_dir, item)
-                old_path = path + ".winecfg"
+                safe_path = path + ".winecfg"
 
-                if os.path.islink(path):
-                    if not restore:
-                        os.unlink(path)
-                elif os.path.isdir(path):
-                    try:
-                        os.rmdir(path)
-                    # We can't delete nonempty dir, so we rename as wine do.
-                    except OSError:
-                        os.rename(path, old_path)
+                self._remove_desktop_folder(path, safe_path)
 
                 # if we want to create a symlink and one is already there, just
                 # skip to the next item.  this also makes sure the elif doesn't
                 # find a dir (isdir only looks at the target of the symlink).
-                if restore and os.path.islink(path):
-                    continue
-                if restore and not os.path.isdir(path):
-                    src_path = get_xdg_entry(DESKTOP_XDG[i])
-                    if not src_path:
-                        logger.error("No XDG entry found for %s, launcher not created", DESKTOP_XDG[i])
-                    else:
-                        os.symlink(src_path, path)
-                    # We don't need all the others process of the loop
-                    continue
-
-                if desktop_dir != user_dir:
-                    try:
-                        src_path = os.path.join(desktop_dir, item)
-                    except TypeError as ex:
-                        # There is supposedly a None value in there
-                        # The current code shouldn't allow that
-                        # Just raise a exception with the values
-                        raise RuntimeError("Missing value desktop_dir=%s or item=%s" % (desktop_dir, item)) from ex
-
-                    os.makedirs(src_path, exist_ok=True)
-                    os.symlink(src_path, path)
+                src_path = get_xdg_entry(DESKTOP_XDG[i])
+                if not src_path:
+                    logger.error("No XDG entry found for %s, launcher not created", DESKTOP_XDG[i])
                 else:
-                    # We use first the renamed dir, otherwise we make it.
-                    if os.path.isdir(old_path):
-                        os.rename(old_path, path)
-                    else:
-                        os.makedirs(path, exist_ok=True)
+                    os.symlink(src_path, path)
+
+            self._set_desktop_integration_assignment(home_dir)
+
+    def _remove_desktop_folder(self, path, safe_path):
+        """Removes the link or directory at 'path'; if it is a non-empty directory
+        this will rename it to 'safe_path' instead of removing it entirely."""
+        if os.path.islink(path):
+            os.unlink(path)
+        elif os.path.isdir(path):
+            try:
+                os.rmdir(path)
+            except OSError:
+                # We can't delete nonempty dir, so we rename as wine do.
+                os.rename(path, safe_path)
+
+    def _get_desktop_integration_assignment(self):
+        try:
+            # If the old tracking file is found, we'll read it, unlink it, and
+            # save the setting in the new form.
+            obsolete_path = os.path.join(self.path, ".lutris_destkop_integration")
+            if os.path.isfile(obsolete_path):
+                with open(obsolete_path, "r", encoding='utf-8') as f:
+                    desktop_dir = f.read()
+                self._set_desktop_integration_assignment(desktop_dir)
+                os.unlink(obsolete_path)
+        except Exception as ex:
+            logger.exception("Unable to read Lutris desktop integration setting: %s", ex)
+
+        settings = get_lutris_directory_settings(self.path)
+        return settings.get("desktop_integration_directory", "")
+
+    def _set_desktop_integration_assignment(self, desktop_dir):
+        set_lutris_directory_settings(self.path, {"desktop_integration_directory": desktop_dir or ""})
 
     def set_crash_dialogs(self, enabled):
         """Enable or diable Wine crash dialogs"""
@@ -235,10 +303,22 @@ class WinePrefixManager:
             self.set_registry_key(path, "WineDesktop", desktop_size)
 
     def set_dpi(self, dpi):
-        """Sets the DPI for WINE to use. None remove the Lutris setting,
-        to leave WINE in control."""
+        """Sets the DPI for WINE to use. None to remove the Lutris setting,
+        and leave WINE in control."""
+
+        # Convert the old hidden file into a 'lutris.json' settings file
+        obsolete_path = os.path.join(self.path, ".lutris_dpi_assignment")
+        try:
+            if os.path.isfile(obsolete_path):
+                with open(obsolete_path, "r", encoding='utf-8') as f:
+                    dpi_assigned = int(f.read())
+                set_lutris_directory_settings(self.path, int(dpi_assigned))
+                os.unlink(obsolete_path)
+        except Exception as ex:
+            logger.exception("Unable to read Lutris assigned DPI: %s", ex)
+
+        settings = get_lutris_directory_settings(self.path)
 
-        assignment_path = os.path.join(self.path, ".lutris_dpi_assignment")
         key_paths = [self.hkcu_prefix + "/Software/Wine/Fonts",
                      self.hkcu_prefix + "/Control Panel/Desktop"]
 
@@ -249,26 +329,27 @@ class WinePrefixManager:
         def is_lutris_dpi_assigned():
             """Check if Lutris assigned the DPI presently found in the registry."""
             try:
-                with open(assignment_path, "r", encoding='utf-8') as f:
-                    assigned_dpi = int(f.read())
+                dpi_assigned = settings.get("dpi_assigned")
+                if dpi_assigned:
+                    dpi_assigned = int(dpi_assigned)
+                else:
+                    return False
             except Exception as ex:
-                logger.exception("Unable to read lutris assigned DPI: %s", ex)
+                logger.exception("Unable to read Lutris assigned DPI: %s", ex)
                 return False
 
             for key_path in key_paths:
-                if assigned_dpi != self.get_registry_key(key_path, "LogPixels"):
+                if dpi_assigned != self.get_registry_key(key_path, "LogPixels"):
                     return False
             return True
 
         if dpi:
             assign_dpi(dpi)
-
-            with open(assignment_path, "w", encoding='utf-8') as f:
-                f.write(str(dpi))
-        elif os.path.isfile(assignment_path):
+            set_lutris_directory_settings(self.path, {"dpi_assigned": dpi})
+        elif settings.get("dpi_assigned"):
             if is_lutris_dpi_assigned():
                 assign_dpi(96)  # reset previous DPI
-            os.remove(assignment_path)
+            set_lutris_directory_settings(self.path, {"dpi_assigned": ""})
 
     def configure_joypads(self):
         """Disables some joypad devices"""
